@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,10 +8,11 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.25 $
+//$Revision: 1.35 $
 
 #include "hk_kdedbdesigner.h"
 #include "hk_kdeformdatasourcedialog.h"
+#include "hk_kderelationdialog.h"
 #include <hk_presentation.h>
 #include <hk_datasource.h>
 #include <hk_dsdatavisible.h>
@@ -37,7 +38,9 @@
 #include <klibloader.h>
 #include <kstandarddirs.h>
 #include <kiconloader.h>
-#include <hk_kderelationdialog.h>
+#include <kapplication.h>
+#include <kconfig.h>
+
 const int designwidth=2000;
 const int designheight=2000;
 
@@ -193,7 +196,10 @@ hk_kdefieldlist::hk_kdefieldlist(hk_kdedatasourceframe* parent):QListBox(parent)
 {
   p_datasourceframe=parent;
   setAcceptDrops(true);
-  setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
+  setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Preferred);
+
+  //setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
+  connect(this,SIGNAL(signal_field_doubleclicked(int, const hk_string&)),p_datasourceframe->designer(),SLOT(slot_field_doubleclicked(int, const hk_string&)));
 }
 
 void hk_kdefieldlist::setFocus(void)
@@ -213,9 +219,18 @@ void hk_kdefieldlist::clearFocus(void)
 	
 
 
-void hk_kdefieldlist::mouseDoubleClickEvent(QMouseEvent*)
+void hk_kdefieldlist::mouseDoubleClickEvent(QMouseEvent* event)
 {
-p_datasourceframe->edit();
+QString field;
+QListBoxItem* item=itemAt(event->pos());
+if (item)
+{
+   field=item->text();
+}
+if (p_datasourceframe->designer()->presentation()->presentationtype()!=hk_presentation::qbe)
+   p_datasourceframe->edit();
+if (!field.isEmpty())
+   emit signal_field_doubleclicked(p_datasourceframe->datasource()->presentationnumber(),field.local8Bit().data());
 }
 
 
@@ -238,7 +253,7 @@ void hk_kdefieldlist::mouseMoveEvent(QMouseEvent* event)
 	QString n;
 	//cerr <<"drag wert="<<index(itemAt(event->pos()))<<endl;
 	n.setNum(p_datasourceframe->datasource()->presentationnumber());
-	n="<VUPN>"+n+"</VUPN>\n<VALUE>"+( itemAt(event->pos())?itemAt(event->pos())->text():"") +"</VALUE>\n";
+	n="<VUPN>"+n+"</VUPN>\n<VALUE>"+( itemAt(event->pos())?itemAt(event->pos())->text():QString("")) +"</VALUE>\n";
 	//cerr<<"dragwert"<<n<<endl;
         QCString cstr =n.utf8();
 	//cerr<"cstr"<<cstr<<endl;
@@ -287,7 +302,7 @@ void hk_kdefieldlist::dropEvent(QDropEvent* event)
 
    hk_kderelationdialog* d = new hk_kderelationdialog(
    						p_datasourceframe->designer()->get_dsframe(masterds),
-         p_datasourceframe);
+         p_datasourceframe,p_datasourceframe->designer());
 
 	((QComboTableItem*)(d->grid->item(0,0)))->setCurrentItem(d->masterindex(QString::fromLocal8Bit(mastertxt.c_str())));
 	if(itemAt(event->pos()))((QComboTableItem*)(d->grid->item(0,1)))->setCurrentItem(d->slaveindex(itemAt(event->pos())->text()));
@@ -386,7 +401,7 @@ hk_kdedatasourceframe::hk_kdedatasourceframe(hk_kdedbdesigner* designer,QWidget*
 bool hk_kdedatasourceframe::eventFilter(QObject* object,QEvent* e)
 {
  QKeyEvent* event=dynamic_cast<QKeyEvent*>(e);
- if (event  && event->key()==Qt::Key_Delete && event->type()==QEvent::KeyRelease)
+ if (event  && event->key()==Qt::Key_Delete && event->type()==QEvent::KeyRelease && datasource())
    {
        if (show_yesnodialog(replace_all(
        		"%1",hk_translate("Remove datasource '%1'?"),
@@ -410,12 +425,14 @@ void hk_kdedatasourceframe::set_fields(void)
 	    cerr <<"hk_kdedatasourceframe::set_fields, NO DATASOURCE SET !!!"<<endl;
 	    return;
 	   } 
-	  list<hk_column*>* cols=datasource()->columns();
+	  list<hk_string>* cols=datasource()->columnnames();
 	  if (!cols) return;
-	  list<hk_column*>::iterator it=cols->begin();
+	  if (p_designer->presentation()->presentationtype()==hk_presentation::qbe)
+	     p_listbox->insertItem("*");
+	  list<hk_string>::iterator it=cols->begin();
 	  while (it!=cols->end())
 	  {
-	  	p_listbox->insertItem(QString::fromLocal8Bit((*it)->name().c_str()));
+	  	p_listbox->insertItem(QString::fromLocal8Bit((*it).c_str()));
 		++it;
 	  }
 	  set_header();
@@ -702,9 +719,12 @@ void hk_kdedbdesigner::set_datasources()
   bool bf=p_private->p_presentation->block_has_changed();
  p_private->p_presentation->set_block_has_changed(true);
  list<hk_datasource*>::iterator it=p_private->p_datasources->begin();
+ hk_kdedatasourceframe* firstframe=NULL;
  while (it!=p_private->p_datasources->end())
  {
-    add_dsframe((*it));
+    hk_kdedatasourceframe* f=add_dsframe((*it));
+    if (!firstframe) firstframe=f;
+    
   ++it;
   }
 
@@ -722,6 +742,7 @@ void hk_kdedbdesigner::set_datasources()
 
 raise_datasources();
    p_private->p_presentation->set_block_has_changed(bf);
+   new_focus(firstframe);
 }
 
 
@@ -741,6 +762,15 @@ hk_kdedatasourceframe* hk_kdedbdesigner::add_dsframe(hk_datasource* ds)
 hk_kdedbrelation* hk_kdedbdesigner::add_relation(hk_kdedatasourceframe* master, hk_kdedatasourceframe* slave)
 {
   if (!master||!slave) return NULL;
+  if (slave&&slave->datasource()&&slave->datasource()->depending_on())
+    {  
+       hk_kdedbrelation* r=get_relation(slave->datasource()->depending_on(),slave->datasource());
+       if (r) delete_relation(r);
+    
+    }
+  
+  
+  
   hk_kdedbrelation* relation=new hk_kdedbrelation(p_private->p_scrollview->viewport());
   p_private->p_scrollview->addChild(relation);
   relation->setAutoMask(true);
@@ -749,6 +779,7 @@ hk_kdedbrelation* hk_kdedbdesigner::add_relation(hk_kdedatasourceframe* master,
   p_private->p_relationlist.insert(p_private->p_relationlist.end(),relation);
   connect(relation,SIGNAL(signal_relation_clicked(hk_kdedbrelation*)),this,SLOT(relation_clicked(hk_kdedbrelation*)));
   relation->show();
+  relation->raise();
   relation->lower();
   slave->set_header();
 emit signal_definition_has_changed();
@@ -759,6 +790,7 @@ emit signal_definition_has_changed();
 
 hk_kdedbrelation* hk_kdedbdesigner::add_relation(hk_datasource* master, hk_datasource* slave)
 {
+  
   return add_relation(get_dsframe(master),get_dsframe(slave));
 }
 
@@ -796,7 +828,6 @@ void hk_kdedbdesigner::delete_datasource(hk_kdedatasourceframe* f)
     	}
    rel=get_relation(f->datasource()->depending_on(),f->datasource());
    if (rel) delete_relation(rel);
-   cerr <<"nach delete"<<f->datasource()->name()<<endl;
   }
   else cerr <<"leere Datasource!!!"<<endl;
   f->deleteLater();
@@ -849,6 +880,7 @@ void hk_kdedbdesigner::add_datasource(void)
     //d->set_datasource(p_slavedatasource->datasource());
 	d->Addbutton->setEnabled(true);
     d->exec();
+    bool changed=d->result()==QDialog::Accepted;
     if (d->clicked_button()==hk_kdeformdatasourcedialog::cb_add)
         {
 	      if (!d->datasource()) return;
@@ -870,11 +902,14 @@ void hk_kdedbdesigner::add_datasource(void)
 	}
 
     delete d;
-emit signal_definition_has_changed();
+if (changed)emit signal_definition_has_changed();
 }
 
 
-
+void hk_kdedbdesigner::slot_field_doubleclicked(int t, const hk_string& c)
+{
+emit signal_field_doubleclicked(t,c);
+}
 
 
 
@@ -1184,7 +1219,8 @@ event->accept();
 
 void hk_kdedbrelation::edit(void)
 {
-    hk_kderelationdialog* d = new hk_kderelationdialog(p_masterdatasource,p_slavedatasource);
+    hk_kderelationdialog* d = new hk_kderelationdialog(p_masterdatasource,p_slavedatasource
+    ,p_masterdatasource->designer(),this);
     d->exec();
   QToolTip::add(this,tooltipfields());
 
@@ -1213,9 +1249,18 @@ p_private=new hk_kdedbdesignerwindowprivate;
    setGeometry(x(),y(),600,500);
    KLibFactory* p_factory= KLibLoader::self()->factory( "libhk_kdedbdesignerpart" );
    p_private->p_part =(KParts::ReadWritePart*) p_factory->create (this, "hk_kdedbdesignerpart", "KParts::ReadWritePart");
-   p_private->p_closeaction=new KAction(i18n("&Close"),loader->loadIcon("exit",KIcon::User),0,this,SLOT(close()),actionCollection(),"closedesigner");
+   p_private->p_closeaction=new KAction(i18n("&Close"),"fileclose",0,this,SLOT(close()),actionCollection(),"closedesigner");
    setCentralWidget(designer());
    createGUI(p_private->p_part);
+     
+     
+  KConfig* c=kapp->config();
+  QRect rect(0,0,500,300);
+  c->setGroup("DatabaseDesigner");
+  QRect g;
+  g=c->readRectEntry("Geometry",&rect);
+  setGeometry(g);
+
 }
 
 hk_kdedbdesignerwindow::~hk_kdedbdesignerwindow()
@@ -1233,7 +1278,10 @@ hk_kdedbdesigner* hk_kdedbdesignerwindow::designer(void) const
 
 void hk_kdedbdesignerwindow::closeEvent(QCloseEvent* e)
 {
-//accept();
+KConfig* c=kapp->config();
+  c->setGroup("DatabaseDesigner");
+  c->writeEntry("Geometry",geometry());
+
 emit signal_closed();
 KParts::MainWindow::closeEvent(e);
 }
