@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.12 $
+//$Revision: 1.22 $
 #include "hk_kdedblistview.h"
 #include <klocale.h>
 #include <kapplication.h>
@@ -73,6 +73,7 @@ void hk_dbviewerdrag::create_draginfo(objecttype type,const QString& object,hk_d
       case ot_database:mystring+="DATABASE";break;
       case ot_table:mystring+="TABLE";break;
       case ot_query:mystring+="QUERY";break;
+      case ot_view:mystring+="VIEW";break;
       case ot_form:mystring+="FORM";break;
       case ot_report:mystring+="REPORT";break;
     }
@@ -160,6 +161,7 @@ hk_kdedblistviewprivate()
 	p_databaseitem=NULL;
 	p_tableitem=NULL;
 	p_queryitem=NULL;
+	p_viewitem=NULL;
 	p_formitem=NULL;
 	p_reportitem=NULL;
 	p_schemadialog=NULL;
@@ -168,6 +170,7 @@ hk_kdedblistviewprivate()
 	}
 KListViewItem* p_databaseitem;
 KListViewItem* p_tableitem;
+KListViewItem* p_viewitem;
 KListViewItem* p_queryitem;
 KListViewItem* p_formitem;
 KListViewItem* p_reportitem;
@@ -245,8 +248,18 @@ void hk_kdedblistview::init_listview()
   
   if (p_private->p_showmode==hk_kdedblistview::sm_all)
   {
+  if (database() && database()->connection()->server_supports(hk_connection::SUPPORTS_VIEWS))
+   {
+     p_private->p_viewitem=new KListViewItem(p_private->p_databaseitem,i18n("Views"));
+     p_private->p_viewitem->setPixmap(0,loader->loadIcon("mail_find",KIcon::Small));
+   }
+   else
+   {
+   p_private->p_viewitem=NULL;
+   }
   p_private->p_tableitem=new KListViewItem(p_private->p_databaseitem,i18n("Tables"));
-  p_private->p_tableitem->setPixmap(0,loader->loadIcon("viewtext",KIcon::Small));
+  p_private->p_tableitem->setPixmap(0,loader->loadIcon("view_text",KIcon::Small));
+  
   } 
   else p_private->p_tableitem=NULL;
   p_private->p_databaseitem->setOpen(true);
@@ -260,6 +273,8 @@ objecttype hk_kdedblistview::current_objecttype()
    if (is_databaseitem(currentItem())) return ot_database;
     else
     if (is_tableitem(currentItem())) return ot_table;
+        else
+        if (is_viewitem(currentItem())) return ot_view;
         else
         if (is_queryitem(currentItem())) return ot_query;
             else
@@ -273,6 +288,8 @@ objecttype hk_kdedblistview::current_objecttype()
 void hk_kdedblistview::set_database(hk_database* db)
 {
   hk_dbvisible::set_database(db);
+  if (db) init_listview();
+  
   set_databasename();
   
 
@@ -289,6 +306,7 @@ if (p_database && p_database->name().size()>0)
     else
     p_private->p_databaseitem->setText(0,i18n("<No database>"));
 set_tables();
+set_views();
 set_queries();
 set_forms();
 set_reports();
@@ -310,6 +328,23 @@ void hk_kdedblistview::set_tables(void)
 }
 
 
+void hk_kdedblistview::set_views(void)
+{
+  
+  if (!p_database||p_private->p_showmode!=hk_kdedblistview::sm_all) return;
+  if (!p_private->p_viewitem) return;
+  p_private->clearList(p_private->p_viewitem);
+  vector<hk_string>* tb=p_database->viewlist();
+  vector<hk_string>::reverse_iterator it;
+  if (tb!=NULL)
+  {
+      for(it=tb->rbegin();it!=tb->rend();it++)
+       new KListViewItem(p_private->p_viewitem,QString::fromLocal8Bit((*it).c_str()));
+  }
+  p_private->p_viewitem->sortChildItems(0,true);
+}
+
+
 void hk_kdedblistview::set_queries(void)
 {
   QString text=i18n("Queries");
@@ -423,6 +458,9 @@ void hk_kdedblistview::item_doubleclicked(QListViewItem* item)
   else
   if (item->parent()==p_private->p_queryitem) 
       emit signal_viewmode_query(item->text(0));
+  else
+  if (p_private->p_viewitem && item->parent()==p_private->p_viewitem) 
+      emit signal_viewmode_view(item->text(0));
 else
   if (item->parent()==p_private->p_formitem) 
       emit signal_viewmode_form(item->text(0));
@@ -435,6 +473,9 @@ else
   else
   if (item==p_private->p_queryitem&& !runtime_only()&&p_database&& p_database->name().size()>0) 
       emit signal_new_query();
+else
+  if (item==p_private->p_viewitem&& !runtime_only()&&p_database&& p_database->name().size()>0) 
+      emit signal_new_view();
 else
   if (item==p_private->p_formitem&& !runtime_only()&&p_database&& p_database->name().size()>0) 
       emit signal_new_form();
@@ -467,6 +508,14 @@ bool hk_kdedblistview::is_queryitem(QListViewItem*item,bool includingheader)
   return false;
 }
 
+bool hk_kdedblistview::is_viewitem(QListViewItem*item,bool includingheader)
+{
+  if (!item || !p_private->p_viewitem) return false;
+  if (includingheader && item==p_private->p_viewitem) return true;
+  if (item->parent()==p_private->p_viewitem) return true;
+  return false;
+}
+
 bool hk_kdedblistview::is_formitem(QListViewItem*item,bool includingheader)
 {
   if (!item) return false;
@@ -495,6 +544,7 @@ bool hk_kdedblistview::is_headeritem(QListViewItem* item)
   return (item==p_private->p_databaseitem ||
           item==p_private->p_tableitem ||
 	  item==p_private->p_queryitem ||
+	  item==p_private->p_viewitem ||
 	  item==p_private->p_formitem ||
 	  item==p_private->p_reportitem 
 	 );
@@ -511,6 +561,12 @@ return is_queryitem(currentItem(),includeheader);
 
 }
 
+bool hk_kdedblistview::is_viewitem(bool includeheader)
+{
+return is_viewitem(currentItem(),includeheader);
+
+}
+
 bool hk_kdedblistview::is_formitem(bool includeheader)
 {
 return is_formitem(currentItem(),includeheader);
@@ -551,6 +607,8 @@ void hk_kdedblistview::new_clicked()
     if (is_reportitem(currentItem(),true)) emit signal_new_report();
     else
     if (is_databaseitem(currentItem())) emit signal_new_database();
+    else
+    if (is_viewitem(currentItem(),true)) emit signal_new_view();
     
 
 }
@@ -563,6 +621,8 @@ void hk_kdedblistview::alter_clicked()
     else
     if (is_queryitem(currentItem())) emit signal_designmode_query(currentItem()->text(0));
     else
+    if (is_viewitem(currentItem())) emit signal_designmode_view(currentItem()->text(0));
+    else
     if (is_formitem(currentItem())) emit signal_designmode_form(currentItem()->text(0));
     else
     if (is_reportitem(currentItem())) emit signal_designmode_report(currentItem()->text(0));
@@ -574,6 +634,8 @@ void hk_kdedblistview::delete_clicked()
     else
     if (is_queryitem(currentItem(),true)) emit signal_delete_query(currentItem()->text(0));
     else
+    if (is_viewitem(currentItem(),true)) emit signal_delete_view(currentItem()->text(0));
+    else
     if (is_formitem(currentItem(),true)) emit signal_delete_form(currentItem()->text(0));
     else
     if (is_reportitem(currentItem(),true)) emit signal_delete_report(currentItem()->text(0));
@@ -590,6 +652,8 @@ void hk_kdedblistview::start_clicked()
     else
     if (is_queryitem(currentItem())) emit signal_viewmode_query(currentItem()->text(0));
     else
+    if (is_viewitem(currentItem())) emit signal_viewmode_view(currentItem()->text(0));
+    else
     if (is_formitem(currentItem())) emit signal_viewmode_form(currentItem()->text(0));
     else
     if (is_reportitem(currentItem())) emit signal_viewmode_report(currentItem()->text(0));
@@ -600,6 +664,8 @@ void    hk_kdedblistview::list_changes(listtype type)
 {
    switch (type)
     {
+        case lt_view : set_views();
+        break;
         case lt_query : set_queries();
         break;
         case lt_form : set_forms();
@@ -634,7 +700,8 @@ bool hk_kdedblistview::acceptDrag(QDropEvent* event) const
     if (xtype=="DATABASE"
         && !(database()->connection()&&database()->connection()->server_supports(hk_connection::SUPPORTS_NEW_DATABASE)))
         cancreate=false;
-   return (event->provides("application/x-hk_filecopy")&&event->source()!=this && cancreate);
+   return (event->provides("application/x-hk_filecopy")&&
+   event->source()!=const_cast<hk_kdedblistview*>(this) && cancreate);
 }
 
 
@@ -675,6 +742,11 @@ bool hk_kdedblistview::decode_action(QMimeSource* event)
         p_private->otype=ot_report;
     }
     else
+    if (p_private->xtype=="VIEW")
+    {
+        p_private->otype=ot_view;
+    }
+    else
     if (p_private->xtype=="QUERY")
     {
         p_private->otype=ot_query;
@@ -709,6 +781,11 @@ bool hk_kdedblistview::execute_action(void)
             copy_table();
             break;
         }
+        case ot_view:
+        {
+            copy_view();
+            break;
+        }
         case ot_database:
         {
             copy_database();
@@ -786,6 +863,40 @@ bool hk_kdedblistview::copy_table(void)
 }
 
 
+bool hk_kdedblistview::copy_view(void)
+{ 
+    if (!database()) return false;
+    hk_connection* con=find_connection();
+    hk_connection* newcon=database()->connection()->drivermanager()->new_connection(p_private->xserver);
+    newcon->set_host(p_private->xhost);
+    newcon->set_tcp_port(localestring2uint(p_private->xtcp));
+    newcon->set_user(p_private->xuser);
+    if (con) newcon->set_password(con->password());
+    newcon->connect();
+    if (!newcon->is_connected())
+    {   
+        delete newcon;
+        return false;
+    }
+    hk_database* db=newcon->new_database(p_private->xdbname);
+    hk_datasource* ds=NULL;
+    if (db)ds= db->new_view(p_private->xname);
+    p_cancelcopying=false;
+    p_progressdialog= new  QProgressDialog();
+    p_progressdialog->resize(300,p_progressdialog->height());
+    connect(p_progressdialog,SIGNAL(cancelled()),this,SLOT(copying_cancelled()));
+    p_progressdialog->show();
+    qApp->processEvents();
+    bool result= database()->copy_view(ds,&hk_kdedblistview::set_progress);
+    delete ds;
+    delete p_progressdialog;
+    p_progressdialog=NULL;
+    newcon->disconnect();
+    delete newcon;
+    return result;
+}
+
+
 void hk_kdedblistview::copying_cancelled()
 {
     p_cancelcopying=true;
