@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.31 $
+//$Revision: 1.36 $
 #include "hk_kdeformdatasourcedialog.h"
 #include <hk_class.h>
 #include <hk_form.h>
@@ -60,6 +60,7 @@ hk_kdeformdatasourcedialog::hk_kdeformdatasourcedialog( hk_presentation* p, QWid
     dependingfieldlist->addColumn(i18n("masterfield"));
     sourcetypefield->insertItem(i18n("table"));
     sourcetypefield->insertItem(i18n("query"));
+    sourcetypefield->insertItem(i18n("view"));
 
     dependingmodefield->insertItem(i18n("standard"));
     dependingmodefield->insertItem(i18n("no handle"));
@@ -120,13 +121,22 @@ void hk_kdeformdatasourcedialog::set_datasourcelist()
         v=db->tablelist();
 	readonlyfield->setEnabled(true);
    }
-    else
+    else     
+    if (sourcetypefield->currentText()==i18n("query"))
+
     {
     v=db->querylist();
 	readonlyfield->setEnabled(false);
         datasourcefield->insertItem("");
 	is_table=false;
    }
+   else
+   
+   {
+     v=db->viewlist();
+     readonlyfield->setEnabled(false);
+     is_table=true;
+   }
 
     vector<hk_string>::iterator it=v->begin();
     while (it!=v->end())
@@ -149,13 +159,21 @@ void hk_kdeformdatasourcedialog::datasource_selected()
     if (p_private->p_presentation==NULL) return;
     hk_database* db=p_private->p_presentation->database();
     if (db==NULL) return;
-    QString txt=i18n("table");
-    bool q=false;
-    if (sourcetypefield->currentText()!=txt) q=true;
-    hk_datasource* ds= db->load_datasource(datasourcefield->currentText().local8Bit().data(),q);
+    QString txt=i18n("query");
+    datasourcetype dt=dt_table;
+  
+    
+    if (sourcetypefield->currentText()==txt) dt=dt_query;
+    else
+    {
+      txt=i18n("view");
+      if (sourcetypefield->currentText()==txt) dt=dt_view;
+    }  
+    
+    hk_datasource* ds= db->load_datasource(datasourcefield->currentText().local8Bit().data(),dt);
     if (ds==NULL) return;
     sqlfield->setEnabled(false);
-    if (q&& datasourcefield->currentText().isEmpty())  sqlfield->setEnabled(true);
+    if (dt==dt_query&& datasourcefield->currentText().isEmpty())  sqlfield->setEnabled(true);
     if (ds->type()!=hk_datasource::ds_table)
     {
         ds->set_temporaryfilter("0=1");
@@ -313,14 +331,20 @@ void hk_kdeformdatasourcedialog::set_datasourceinfos(bool existingdatasource)
 
     if (p_private->p_presentation==NULL)  return;
 
-    QString txt=i18n("table");
-    bool q=false;
-    if (sourcetypefield->currentText()!=txt) q=true;
+    QString txt=i18n("query");
+    datasourcetype dt=dt_table;
+    if (sourcetypefield->currentText()==txt) dt=dt_query;
+    else
+    {
+     txt=i18n("view");
+     if (sourcetypefield->currentText()==txt) dt=dt_view;
+    }
+    
     hk_datasource* d=p_private->p_datasource;
 
     if (!existingdatasource)
     {
-        long  id =p_private->p_presentation->new_datasource(datasourcefield->currentText().local8Bit().data(),q);
+        long  id =p_private->p_presentation->new_datasource(datasourcefield->currentText().local8Bit().data(),dt);
         d=p_private->p_presentation->get_datasource(id);
 	p_private->p_datasource=d;
     }
@@ -396,12 +420,14 @@ void hk_kdeformdatasourcedialog::set_datasource(hk_datasource* d)
     sortingfield->setText(QString::fromLocal8Bit (p_private->p_datasource->sorting().c_str()));
     readonlyfield->setChecked(p_private->p_datasource->is_readonly());
 
-    if (p_private->p_datasource->type()==hk_data::ds_table) sourcetypefield->setCurrentItem(0);
-    else
+    switch (p_private->p_datasource->type())
     {
-        sourcetypefield->setCurrentItem(1);
-        set_datasourcelist();
+    case hk_data::ds_table: sourcetypefield->setCurrentItem(0);break;
+    case hk_data::ds_query: sourcetypefield->setCurrentItem(1);break;
+    default: sourcetypefield->setCurrentItem(2);
     }
+    
+    set_datasourcelist();
     int c=0;
     QString t=QString::fromLocal8Bit (p_private->p_datasource->name().c_str());
     while (c<datasourcefield->count())
@@ -507,10 +533,17 @@ void hk_kdeformdatasourcedialog::set_displaytype(enum_displaytype d)
 			dependingmodelabel->hide();
 			Frame17->hide();
 			Frame6->hide();
-			sourcetypefield->setEnabled(false);
+			sourcetypefield->setEnabled(true);
 			readonlyfield->hide();
     			break;
   }
+  
+    sourcetypefield->clear();
+    sourcetypefield->insertItem(i18n("table"));
+    if (d!=d_query)sourcetypefield->insertItem(i18n("query"));
+    sourcetypefield->insertItem(i18n("view"));
+
+  
   p_private->p_displaytype=d;
   adjustSize();
 }
@@ -528,7 +561,7 @@ void hk_kdeformdatasourcedialog::set_edittype(enum_edittype t)
    {
      case  e_all:
      case  e_add:
-     			sourcetypefield->setEnabled(p_private->p_displaytype!=d_query);
+     			sourcetypefield->setEnabled(true);
 			datasourcefield->setEnabled(true);
      			break;
      case  e_modify:
