@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.89 $
+//$Revision: 1.101 $
 #include "hk_kdeproperty.h"
 #include "hk_visible.h"
 #include "hk_dsvisible.h"
@@ -19,17 +19,26 @@
 #include "hk_kdelabel.h"
 #include "hk_kdereportconditiondialog.h"
 #include "hk_kdeinterpreterdialog.h"
+#include "hk_kdesubformdialog.h"
 #include <hk_database.h>
 #include <hk_colour.h>
 #include <hk_button.h>
+#include "hk_kdesubform.h"
+#include "hk_kdefilterdialog.h"
+#include "hk_kdecomboboxtextlist.h"
+
 #include <qcombobox.h>
 #include <qspinbox.h>
 #include <qfontdatabase.h>
 #include <qlineedit.h>
+#include <qscrollview.h>
+#include <qsizepolicy.h>
+#include <qmultilineedit.h>
 #include <qpushbutton.h>
 #include <qlabel.h>
 #include <qtabwidget.h>
 #include <qtoolbutton.h>
+#include <qtooltip.h>
 #include <qlayout.h>
 #include <qcolor.h>
 #include <hk_datasource.h>
@@ -44,12 +53,48 @@
  *  The dialog will by default be modeless, unless you set 'modal' to
  *  TRUE to construct a modal dialog.
  */
-hk_kdeproperty::hk_kdeproperty( hk_kdesimpleform* parent,  const char* name, bool modal, WFlags fl )
-: hk_kdepropertyeditorbase( parent, name, modal, fl )  , hk_class()
+hk_kdeproperty::hk_kdeproperty( hk_kdesimpleform* parent,  const char* name, WFlags fl )
+: hk_kdepropertyeditorbase( parent, name,  fl )  , hk_class()
 {
 #ifdef HK_DEBUG
+//wanna_debug(true);
     hkdebug("hk_kdeproperty::hk_kdeproperty");
 #endif
+    
+    tab->removePage(tab_2);
+    
+    p_datascrollview=new QScrollView(tab);
+    tab->insertTab(p_datascrollview,"");
+    tab_2->reparent(p_datascrollview->viewport(),QPoint(0,0));
+    p_datascrollview->addChild(tab_2);
+    
+
+    tab->removePage(tab_3);
+    
+    p_formatscrollview=new QScrollView(tab);
+    tab->insertTab(p_formatscrollview,"");
+    tab_3->reparent(p_formatscrollview->viewport(),QPoint(0,0));
+    p_formatscrollview->addChild(tab_3);
+    
+
+    tab->removePage(frametab);
+    
+    p_framescrollview=new QScrollView(tab);
+    tab->insertTab(p_framescrollview,"");
+    frametab->reparent(p_framescrollview->viewport(),QPoint(0,0));
+    p_framescrollview->addChild(frametab);
+    
+
+    tab->removePage(tab_4);
+    
+    p_actionscrollview=new QScrollView(tab);
+    tab->insertTab(p_actionscrollview,"");
+    tab_4->reparent(p_actionscrollview->viewport(),QPoint(0,0));
+    p_actionscrollview->addChild(tab_4);
+    
+
+    
+        
     QString no=i18n("no");
     QString yes=i18n("yes");
     p_form=parent;
@@ -75,12 +120,15 @@ hk_kdeproperty::hk_kdeproperty( hk_kdesimpleform* parent,  const char* name, boo
     actionfield->insertItem(i18n("Delete row"));
     actionfield->insertItem(i18n("Store row"));
     actionfield->insertItem(i18n("Execute actionquery"));
+    actionfield->insertItem(i18n("Close application"));
+    actionfield->insertItem(i18n("Open view"));
     //actionfield->insertItem(i18n("Execute script"));
 
     actionfield->setCurrentItem(0);
-    tab->changeTab(tab_3,i18n("&Format"));
-    tab->changeTab(tab_2,i18n("&Data"));
-    tab->changeTab(tab_4,i18n("&Action"));
+    tab->changeTab(p_formatscrollview,i18n("&Format"));
+    tab->changeTab(p_datascrollview,i18n("&Data"));
+    tab->changeTab(p_actionscrollview,i18n("&Action"));
+    tab->changeTab(p_framescrollview,i18n("F&rame"));
     labellabel->setText(i18n("Label:"));
     xlabel->setText(i18n("X:"));
     ylabel->setText(i18n("Y:"));
@@ -104,6 +152,8 @@ hk_kdeproperty::hk_kdeproperty( hk_kdesimpleform* parent,  const char* name, boo
     showmaximizedfield->insertItem(yes);
     separatorfield->insertItem(yes);
     separatorfield->insertItem(no);
+    usetextlistfield->insertItem(yes);
+    usetextlistfield->insertItem(no);
 
     readonlyfield->insertItem(yes);
     readonlyfield->insertItem(no);
@@ -143,6 +193,39 @@ hk_kdeproperty::hk_kdeproperty( hk_kdesimpleform* parent,  const char* name, boo
     connect( p_backgroundcolourbutton, SIGNAL( changed(const QColor&) ), this, SLOT( format_changes() ) );
     settxt=i18n("set");
     notsettxt=i18n("not set");
+    
+    labelfield->installEventFilter(this);
+    defaultfield->installEventFilter(this);
+    identifierfield->installEventFilter(this);
+    tooltipfield->installEventFilter(this);
+    QString info=i18n("Press <F2> for a larger editor window");
+    QToolTip::add(labelfield,info);
+    QToolTip::add(defaultfield,info);
+    QToolTip::add(identifierfield,info);
+    QToolTip::add(tooltipfield,info);
+
+    QSize s=tab_3->geometry().size();
+    if (frametab->width()>s.width())s.setWidth(frametab->width());
+    if (tab_2->width()>s.width())s.setWidth(tab_2->width());
+    if (tab_4->width()>s.width())s.setWidth(tab_4->width());
+    
+    if (frametab->height()>s.height())s.setHeight(frametab->height());
+    if (tab_2->height()>s.height())s.setHeight(tab_2->height());
+    if (tab_4->height()>s.height())s.setHeight(tab_4->height());
+    
+    QSizePolicy sp=QSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
+    frametab->setSizePolicy(sp);
+    tab_2->setSizePolicy(sp);
+    tab_3->setSizePolicy(sp);
+    tab_4->setSizePolicy(sp);
+    frametab->setMinimumSize(s);
+    tab_2->setMinimumSize(s);
+    tab_3->setMinimumSize(s);
+    tab_4->setMinimumSize(s);
+
+
+
+
 }
 
 
@@ -218,6 +301,7 @@ void hk_kdeproperty::data_changes()
 #ifdef HK_DEBUG
     hkdebug("hk_kdeproperty::data_changes()");
 #endif
+
     if (p_visible==NULL) return;
     switch (p_visible->type())
     {
@@ -226,12 +310,15 @@ void hk_kdeproperty::data_changes()
         case hk_visible::memo:;
         case hk_visible::combobox:;
         set_objectdsdatavisible();
-
+        case hk_visible::subform:
+				set_objectsubform();
         case hk_visible::form:
         case hk_visible::grid:
         case hk_visible::button:
         case hk_visible::rowselector:
             set_objectdsvisible();
+	    break;
+	    
         default:;
 
     }
@@ -247,6 +334,14 @@ if (p_visible->identifier()!=t) p_visible->set_identifier(t);
 
 }
 
+void hk_kdeproperty::tooltipfield_changed()
+{
+hk_string t=tooltipfield->text().local8Bit().data();
+if (p_visible->tooltip()!=t) p_visible->set_tooltip(t);
+
+}
+
+
 /*
  * public slot
  */
@@ -287,6 +382,14 @@ void hk_kdeproperty::set_object(hk_visible* v)
     hkdebug("hk_kdeproperty::set_object");
 #endif
     p_visible=v;
+    
+    if (p_visible)
+    { 
+      QString num;
+      num.setNum(p_visible->presentationnumber());
+      idfield->setText(num);
+    }  
+    
     tab_2->setEnabled(false);
     tab_3->setEnabled(false);
     tab_4->setEnabled(true);
@@ -312,7 +415,9 @@ void hk_kdeproperty::set_object(hk_visible* v)
     listcolumnfield->setEnabled(false);
     viewcolumnfield->setEnabled(false);
     combomodefield->setEnabled(false);
+    combomodefield->setEnabled(false);
     listdatasourcefield->setEnabled(false);
+    usetextlistfield->setEnabled(false);
     listdatasourcebutton->setEnabled(false);
     separatorfield->setEnabled(false);
     alignmentfield->setEnabled(false);
@@ -341,8 +446,11 @@ void hk_kdeproperty::set_object(hk_visible* v)
     afterdeleteactionbutton->setEnabled(false);
     beforeinsertactionbutton->setEnabled(false);
     afterinsertactionbutton->setEnabled(false);
+    subformfield->setEnabled(false);
+    subformbutton->setEnabled(false);
 
     set_visible();
+    
     switch (p_visible->type())
     {
         case hk_visible::grid:            set_dsvisible();
@@ -360,22 +468,33 @@ void hk_kdeproperty::set_object(hk_visible* v)
             combomodefield->blockSignals(true);
 	    combomodefield->clear();
 	    combomodefield->insertItem(i18n("Combo"));
+	    combomodefield->insertItem(i18n("Combo (nonwritable)"));
             combomodefield->insertItem(i18n("Selector"));
             combomodefield->blockSignals(false);
+	    usetextlistfield->setEnabled(true);
+            use_textlist_changed();
 
-        combomodefield->setEnabled(true);
+
+        	combomodefield->setEnabled(true);
         case hk_visible::lineedit:;
         case hk_visible::memo:        alignmentfield->setEnabled(true);
-        digitfield->setEnabled(true);
-        separatorfield->setEnabled(true);
-        case hk_visible::boolean:;
-        defaultfield->setEnabled(true);
-        columnfield->setEnabled(true);
-        set_dsvisible();
-        set_dsdatavisible();
-        tab_2->setEnabled(true);
+        	digitfield->setEnabled(true);
+        	separatorfield->setEnabled(true);
+        
+	case hk_visible::boolean:;
+        	defaultfield->setEnabled(true);
+        	columnfield->setEnabled(true);
+        	set_dsvisible();
+        	set_dsdatavisible();
+        	tab_2->setEnabled(true);
 
         break;
+	case hk_visible::subform:
+	                subformfield->setEnabled(true);
+			set_dsvisible();
+			set_subform();
+			tab_2->setEnabled(true);
+			break;
 
         case  hk_visible::form:
             {combomodefield->setEnabled(true);
@@ -409,7 +528,8 @@ void hk_kdeproperty::set_object(hk_visible* v)
 	actionfield->setEnabled(true);
         readonlyfield->setEnabled(false);
         break;
-        case  hk_visible::textlabel:  tab_2->setEnabled(true);
+        
+	case  hk_visible::textlabel:  tab_2->setEnabled(true);
         datasourcefield->setEnabled(false);
         alignmentfield->setEnabled(true);
         columnfield->setEnabled(false);
@@ -463,6 +583,10 @@ void hk_kdeproperty::set_visible(void)
     identifierfield->setText(QString::fromLocal8Bit (p_visible->identifier().c_str()));
     identifierfield->blockSignals(false);
 
+    tooltipfield->blockSignals(true);
+    tooltipfield->setText(QString::fromLocal8Bit (p_visible->tooltip().c_str()));
+    tooltipfield->blockSignals(false);
+
     p_foregroundcolourbutton->blockSignals(true);
     hk_colour c=p_visible->foregroundcolour();
     QColor qcol(c.red(),c.green(),c.blue());
@@ -493,6 +617,10 @@ void hk_kdeproperty::set_visible(void)
     alignmentfield->setCurrentItem(a);
     alignmentfield->blockSignals(false);
     onpushactionbutton->setText(p_visible->on_click_action().size()>0?settxt:notsettxt);
+    ongetfocusbutton->setText(p_visible->on_getfocus_action().size()>0?settxt:notsettxt);
+    onloosefocusbutton->setText(p_visible->on_loosefocus_action().size()>0?settxt:notsettxt);
+    onkeybutton->setText(p_visible->on_key_action().size()>0?settxt:notsettxt);
+    onpushactionbutton->setText(p_visible->on_click_action().size()>0?settxt:notsettxt);
     ondoubleclickactionbutton->setText(p_visible->on_doubleclick_action().size()>0?settxt:notsettxt);
     onopenactionbutton->setText(p_visible->on_open_action().size()>0?settxt:notsettxt);
     oncloseactionbutton->setText(p_visible->on_close_action().size()>0?settxt:notsettxt);
@@ -607,8 +735,16 @@ void hk_kdeproperty::set_dsdatavisible(void)
     viewcolumnfield->setEditText(QString::fromLocal8Bit (combo->viewcolumnname().c_str()));
     viewcolumnfield->blockSignals(false);
     combomodefield->blockSignals(true);
-    combomodefield->setCurrentItem((combo->mode()==hk_dscombobox::selector)?1:0);
+    int p=0;
+    switch (combo->mode())
+    {
+     case hk_dscombobox::combo : p=0;break;
+     case hk_dscombobox::combo_noedit : p=1;break;
+     case hk_dscombobox::selector : p=2;break;
+    }
+    combomodefield->setCurrentItem(p);
     combomodefield->blockSignals(false);
+    
     if (combo->mode()==hk_dscombobox::selector)
     {
         listdatasourcefield->setEnabled(false);
@@ -623,6 +759,11 @@ void hk_kdeproperty::set_dsdatavisible(void)
 
     }
 
+     usetextlistfield->blockSignals(true);
+     usetextlistfield->setCurrentItem(combo->use_textlist()?0:1);
+     usetextlistbutton->setEnabled(combo->use_textlist());
+     usetextlistfield->blockSignals(false);
+
 }
 
 void hk_kdeproperty::set_labelvisible(void)
@@ -665,6 +806,43 @@ void hk_kdeproperty::set_gridvisible(void)
     if (p_visible==NULL)return;
 }
 
+void hk_kdeproperty::set_subform(void)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_kdeproperty::set_subform");
+#endif
+
+    if (!p_form) return;
+    hk_subform* subform=dynamic_cast<hk_subform*>(p_visible);
+    if (!subform) return;
+    subformfield->blockSignals(true);
+    subformfield->clear();
+    subformfield->insertItem("");
+
+    vector<hk_string>* dslist = p_form->database()->formlist();
+    if (dslist==NULL)
+    {
+        subformfield->blockSignals(false);
+        return;
+    }
+    vector<hk_string>::iterator it=dslist->begin();
+    int item=1;
+    while (it!=dslist->end())
+    {
+        QString name= QString::fromLocal8Bit ((*it).c_str());
+        if ((*it)!=p_form->hk_form::name())
+	   subformfield->insertItem(name);
+	   else --item;
+        if (subform->name()==(*it)) 
+	   subformfield->setCurrentItem(item);
+        it++; ++item;
+    }
+    subformfield->blockSignals(false);
+subformbutton->setEnabled(!subformfield->currentText().isEmpty()
+&&!datasourcefield->currentText().isEmpty()
+);
+}
+
 
 void hk_kdeproperty::set_button(void)
 {
@@ -688,11 +866,13 @@ void hk_kdeproperty::set_button(void)
     if (i==5||i==6)
     {
         liste=p_form->database()->reportlist();
-        conditionbutton->setEnabled(true);
 
     }
     else
-    if (i==1||i==2)  liste=p_form->database()->formlist();
+    if (i==1||i==2) 
+      {
+       liste=p_form->database()->formlist();
+      } 
         else
             liste=p_form->database()->tablelist();
     vector<hk_string>::iterator it =  liste->begin();
@@ -706,6 +886,7 @@ void hk_kdeproperty::set_button(void)
         }
         it++;  current++;
     }
+ conditionbutton->setEnabled((i==1||i==5||i==6)&&!objectfield->currentText().isEmpty());
 
 }
 
@@ -775,7 +956,7 @@ void hk_kdeproperty::set_objectdsvisible(void)
         }
         if (combo!=NULL)
         {
-            if (combomodefield->currentItem()==1)
+            if (combomodefield->currentItem()==2)
             {
                 listdatasourcefield->setEnabled(false);
                 listcolumnfield->setEnabled(false);
@@ -808,7 +989,17 @@ void hk_kdeproperty::set_objectdsvisible(void)
     }
     if (combo)
         {
-	combo->set_mode(combomodefield->currentItem()==1?hk_dscombobox::selector:hk_dscombobox::combo);
+	hk_dscombobox::enum_mode m;
+	switch(combomodefield->currentItem())
+	{
+	  case 1 : m=hk_dscombobox::combo_noedit;break;
+	  case 2 : m=hk_dscombobox::selector;break;
+	  default: m=hk_dscombobox::combo;break;
+	  
+	}
+	combo->set_mode(m);
+	combo->set_use_textlist(usetextlistfield->currentItem()==0);
+	use_textlist_changed();
 	}
 }
 
@@ -833,6 +1024,23 @@ void hk_kdeproperty::set_objectdsdatavisible(void)
 }
 
 
+void hk_kdeproperty::set_objectsubform(void)
+{cerr<<"set_objectsubform"<<endl;
+#ifdef HK_DEBUG
+    hkdebug("hk_kdeproperty::set_objectsubform");
+#endif
+hk_subform* subform= dynamic_cast<hk_subform*>(p_visible);
+if (!subform) return;
+subform->set_name(subformfield->currentText().local8Bit().data());
+subformbutton->setEnabled(!subformfield->currentText().isNull());
+
+
+
+}
+
+
+
+
 void hk_kdeproperty::set_objectaction(void)
 {
 #ifdef HK_DEBUG
@@ -849,8 +1057,8 @@ void hk_kdeproperty::set_objectaction(void)
         (showmaximizedfield->currentItem()==1?true:false )
         );
 
-    int i = bt->action(); //cerr <<"set_objectaction i="<<endl;
-    conditionbutton->setEnabled(i==4||i==5);
+    int i = bt->action(); cerr <<"set_objectaction i="<<i<<endl;
+    conditionbutton->setEnabled(!objectfield->currentText().isEmpty()&&(i==0||i==4||i==5));
 
 }
 
@@ -1085,7 +1293,7 @@ void hk_kdeproperty::set_viewcolumnvalues()
     viewcolumnfield->blockSignals(false);
 
     QString f;
-    if (combomodefield->currentItem()==1&& p_visible&&p_visible->type()==hk_visible::combobox)
+    if (combomodefield->currentItem()==2&& p_visible&&p_visible->type()==hk_visible::combobox)
         f=datasourcefield->currentText();
     else
         f=listdatasourcefield->currentText();
@@ -1111,8 +1319,9 @@ void hk_kdeproperty::set_viewcolumnvalues()
 
 void hk_kdeproperty::conditionbutton_clicked()
 {
+int i = actionfield->currentItem()-1; cerr <<"condi actionobjectlist i="<<i<<endl;
     hk_kdereportconditiondialog* d = new hk_kdereportconditiondialog(this,0);
-    d->set_values(dynamic_cast <hk_button*>(p_visible),p_form,objectfield->currentText().local8Bit().data());
+    d->set_values(dynamic_cast <hk_button*>(p_visible),p_form,objectfield->currentText().local8Bit().data(),(i==0?hk_kdereportconditiondialog::form:hk_kdereportconditiondialog::report));
     d->exec();
     delete d;
 }
@@ -1134,6 +1343,8 @@ void hk_kdeproperty::pushactionbutton_clicked(int rownumber,const hk_string& war
     }
 
    set_visible();
+   set_dsvisible();
+   set_dsdatavisible();
     delete d;
 
 
@@ -1157,6 +1368,8 @@ void hk_kdeproperty::doubleclickactionbutton_clicked(int rownumber,const hk_stri
     }
 
    set_visible();
+   set_dsvisible();
+   set_dsdatavisible();
     delete d;
 
 
@@ -1175,6 +1388,8 @@ void hk_kdeproperty::openactionbutton_clicked(int rownumber,const hk_string& war
     }
 
    set_visible();
+   set_dsvisible();
+   set_dsdatavisible();
     delete d;
 
 
@@ -1196,6 +1411,8 @@ void hk_kdeproperty::closeactionbutton_clicked(int rownumber,const hk_string& wa
       p_visible->set_on_close_action(d->code());
     }
    set_visible();
+   set_dsvisible();
+   set_dsdatavisible();
 
     delete d;
 
@@ -1220,7 +1437,9 @@ void hk_kdeproperty::beforedeletebutton_clicked(int rownumber,const hk_string& w
       ds->set_before_delete_action(d->code());
     }
 
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
 
     delete d;
 
@@ -1245,7 +1464,9 @@ void hk_kdeproperty::afterdeletebutton_clicked(int rownumber,const hk_string& wa
       ds->set_after_delete_action(d->code());
     }
 
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
 
     delete d;
 
@@ -1269,7 +1490,9 @@ void hk_kdeproperty::beforeinsertbutton_clicked(int rownumber,const hk_string& w
       ds->set_before_insert_action(d->code());
     }
 
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
 
     delete d;
 
@@ -1294,7 +1517,9 @@ void hk_kdeproperty::afterinsertbutton_clicked(int rownumber,const hk_string& wa
       ds->set_after_insert_action(d->code());
     }
 
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
 
     delete d;
 
@@ -1318,7 +1543,9 @@ void hk_kdeproperty::beforeupdatebutton_clicked(int rownumber,const hk_string& w
       ds->set_before_update_action(d->code());
     }
 
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
 
     delete d;
 
@@ -1341,7 +1568,9 @@ void hk_kdeproperty::afterupdatebutton_clicked(int rownumber,const hk_string& wa
     {
       ds->set_after_update_action(d->code());
     }
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
 
 
     delete d;
@@ -1366,7 +1595,9 @@ void hk_kdeproperty::beforerowchangebutton_clicked(int rownumber,const hk_string
       ds->set_before_row_change_action(d->code());
     }
 
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
 
     delete d;
 
@@ -1390,7 +1621,9 @@ void hk_kdeproperty::afterrowchangebutton_clicked(int rownumber,const hk_string&
       ds->set_after_row_change_action(d->code());
     }
 
+   set_visible();
    set_dsvisible();
+   set_dsdatavisible();
     delete d;
 
 }
@@ -1400,7 +1633,107 @@ void hk_kdeproperty::afterrowchangebutton_clicked(void)
 afterrowchangebutton_clicked(0,"");
 }
 
+void hk_kdeproperty::ongetfocusbutton_clicked(int rownumber,const hk_string& warningmessage)
+{
+  hk_dsvisible* ds=dynamic_cast<hk_dsvisible*>(p_visible);
+  if (!ds) return;
+    hk_kdeinterpreterdialog* d = new hk_kdeinterpreterdialog(0,0,Qt::WShowModal);
+    d->set_code(ds->on_getfocus_action(),false);
+    d->set_caption(p_visible,"on_getfocus_action");
+    int r=d->exec(rownumber,warningmessage);
+    if (r==hk_kdeinterpreterdialog::Accepted&&d->has_changed())
+    {
+      ds->set_on_getfocus_action(d->code());
+    }
 
+   set_visible();
+   set_dsvisible();
+   set_dsdatavisible();
+
+    delete d;
+
+
+}
+
+void hk_kdeproperty::ongetfocusbutton_clicked(void)
+{
+ongetfocusbutton_clicked(0,"");
+}
+
+void hk_kdeproperty::onloosefocusbutton_clicked(int rownumber,const hk_string& warningmessage)
+{
+  hk_dsvisible* ds=dynamic_cast<hk_dsvisible*>(p_visible);
+  if (!ds) return;
+    hk_kdeinterpreterdialog* d = new hk_kdeinterpreterdialog(0,0,Qt::WShowModal);
+    d->set_code(ds->on_loosefocus_action(),false);
+    d->set_caption(p_visible,"on_loosefocus_action");
+    int r=d->exec(rownumber,warningmessage);
+    if (r==hk_kdeinterpreterdialog::Accepted&&d->has_changed())
+    {
+      ds->set_on_loosefocus_action(d->code());
+    }
+
+   set_visible();
+   set_dsvisible();
+   set_dsdatavisible();
+
+    delete d;
+
+
+}
+
+void hk_kdeproperty::onloosefocusbutton_clicked(void)
+{
+onloosefocusbutton_clicked(0,"");
+}
+
+
+ 
+void hk_kdeproperty::onkeybutton_clicked(int rownumber,const hk_string& warningmessage)
+{
+  hk_dsvisible* ds=dynamic_cast<hk_dsvisible*>(p_visible);
+  if (!ds) return;
+    hk_kdeinterpreterdialog* d = new hk_kdeinterpreterdialog(0,0,Qt::WShowModal);
+    d->set_code(ds->on_key_action(),false);
+    d->set_caption(p_visible,"on_key_action");
+    int r=d->exec(rownumber,warningmessage);
+    if (r==hk_kdeinterpreterdialog::Accepted&&d->has_changed())
+    {
+      ds->set_on_key_action(d->code());
+    }
+
+   set_visible();
+   set_dsvisible();
+   set_dsdatavisible();
+
+    delete d;
+
+
+}
+
+void hk_kdeproperty::onkeybutton_clicked(void)
+{
+onkeybutton_clicked(0,"");
+}
+
+
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
+ 
  void hk_kdeproperty::border_changes()
  {
    if (!p_visible) return;
@@ -1422,3 +1755,69 @@ return p_visible;
 }
 
 
+void hk_kdeproperty::subformbutton_clicked()
+{
+  hk_subform* subf=dynamic_cast<hk_subform*>(p_visible);
+  if (!subf) return;
+  hk_kdesubformdialog* d=new hk_kdesubformdialog(subf);
+    if (d->exec()==QDialog::Accepted)
+       {
+
+       }
+
+    delete d;
+}
+
+
+void hk_kdeproperty::use_editor(QLineEdit* e)
+{
+  if (!e) return;
+    hk_kdefilterdialog* f = new hk_kdefilterdialog(hk_kdefilterdialog::editor,this,0,true);
+    
+    f->textfield->setText(e->text());
+    if (f->exec()==QDialog::Accepted)
+    {
+        e->setText(f->textfield->text());
+    };
+
+    delete f;
+
+}
+
+bool hk_kdeproperty::eventFilter(QObject* object,QEvent* event)
+{
+
+   QLineEdit* edit=dynamic_cast<QLineEdit*>(object);
+   
+   if (edit&&event->type()==QEvent::KeyPress)
+   {
+     QKeyEvent*ev=(QKeyEvent*)event;
+     if (ev->key()==Key_F2)
+       use_editor(edit);
+   } 
+  
+   return hk_kdepropertyeditorbase::eventFilter(object,event);
+}
+
+void hk_kdeproperty::use_textlist_changed()
+{
+  bool e=usetextlistfield->currentItem()==0;
+  usetextlistbutton->setEnabled(e);
+  combomodefield->setEnabled(!e);
+  listcolumnfield->setEnabled(!e);
+  viewcolumnfield->setEnabled(!e);
+  
+}
+
+void hk_kdeproperty::use_textlistbutton_clicked()
+{
+  hk_dscombobox* c=dynamic_cast<hk_dscombobox*>(p_visible);
+  if (!c) return;
+    hk_kdecomboboxtextlist* d = new hk_kdecomboboxtextlist(this,c);
+    d->exec();
+    delete d;
+
+}
+
+
+
