@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,44 +8,84 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.50 $
+//$Revision: 1.56 $
 #include "hk_kdereportproperty.h"
 #include "hk_kdesimplereport.h"
 #include "hk_kdesubreportdialog.h"
 #include "hk_kdereportsection.h"
 #include "hk_kdeinterpreterdialog.h"
 #include "hk_kdespinbox.h"
+#include "hk_kdeformdatasourcedialog.h"
+#include "hk_kdereportdata.h"
+#include "hk_kdefilterdialog.h"
+
 #include <hk_datasource.h>
 #include <hk_database.h>
 #include <hk_colour.h>
-#include <qcombobox.h>
-#include <qlineedit.h>
-#include <qspinbox.h>
-#include <qlabel.h>
-#include <qpushbutton.h>
-#include <qtoolbutton.h>
-#include <qfontdatabase.h>
-#include <qlayout.h>
-#include <hk_kdeformdatasourcedialog.h>
-#include <hk_kdereportdata.h>
 #include <hk_visible.h>
 #include <hk_reportdata.h>
 #include <hk_reportsection.h>
 #include <hk_reportsectionpair.h>
+
+#include <qcombobox.h>
+#include <qlineedit.h>
+#include <qspinbox.h>
+#include <qlabel.h>
+#include <qlineedit.h>
+#include <qmultilineedit.h>
+#include <qpushbutton.h>
+#include <qtoolbutton.h>
+#include <qfontdatabase.h>
+#include <qscrollview.h>
+#include <qlayout.h>
 #include <qtabwidget.h>
 #include <qcolor.h>
+#include <qstringlist.h>
+#include <qtooltip.h>
+
 #include <klocale.h>
 #include <kcolorbutton.h>
-#include <qstringlist.h>
 
-hk_kdereportproperty::hk_kdereportproperty( hk_kdesimplereport* parent,  const char* name, bool modal, WFlags fl )
-: hk_kdereportpropertybase( parent, name, modal, fl )  , hk_class()
+hk_kdereportproperty::hk_kdereportproperty( hk_kdesimplereport* parent,  const char* name, WFlags fl )
+: hk_kdereportpropertybase( parent, name,  fl )  , hk_class()
 {
 
 #ifdef HK_DEBUG
     hkdebug("hk_kdereportproperty::hk_kdereportproperty()");
 //wanna_debug(true);
 #endif
+    tab->removePage(datatab);
+    
+    p_datascrollview=new QScrollView(tab);
+    tab->insertTab(p_datascrollview,"");
+    datatab->reparent(p_datascrollview->viewport(),QPoint(0,0));
+    p_datascrollview->addChild(datatab);
+    
+
+    tab->removePage(formattab);
+    
+    p_formatscrollview=new QScrollView(tab);
+    tab->insertTab(p_formatscrollview,"");
+    formattab->reparent(p_formatscrollview->viewport(),QPoint(0,0));
+    p_formatscrollview->addChild(formattab);
+    
+
+    tab->removePage(frametab);
+    
+    p_framescrollview=new QScrollView(tab);
+    tab->insertTab(p_framescrollview,"");
+    frametab->reparent(p_framescrollview->viewport(),QPoint(0,0));
+    p_framescrollview->addChild(frametab);
+    
+
+    tab->removePage(actiontab);
+    
+    p_actionscrollview=new QScrollView(tab);
+    tab->insertTab(p_actionscrollview,"");
+    actiontab->reparent(p_actionscrollview->viewport(),QPoint(0,0));
+    p_actionscrollview->addChild(actiontab);
+    
+    
     pageformatfield=NULL;
     orientationfield=NULL;
     firstpagenumberfield=NULL;
@@ -146,7 +186,7 @@ hk_kdereportproperty::hk_kdereportproperty( hk_kdesimplereport* parent,  const c
 
     formattabLayout->addWidget( p_foregroundcolourlabel, 13, 0 );
 
-    p_foregroundcolourbutton = new KColorButton( FALSE, formattab, "foregroundcolourfield" );
+    p_foregroundcolourbutton = new KColorButton(  formattab, "foregroundcolourfield" );
     p_foregroundcolourbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, separatorfield->sizePolicy().hasHeightForWidth() ) );
     p_foregroundcolourbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, separatorfield->sizePolicy().hasHeightForWidth() ) );
 
@@ -159,7 +199,7 @@ hk_kdereportproperty::hk_kdereportproperty( hk_kdesimplereport* parent,  const c
 
     formattabLayout->addWidget( p_backgroundcolourlabel, 14, 0 );
 
-    p_backgroundcolourbutton = new KColorButton( FALSE, formattab, "backgroundcolourfield" );
+    p_backgroundcolourbutton = new KColorButton( formattab, "backgroundcolourfield" );
     p_backgroundcolourbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, separatorfield->sizePolicy().hasHeightForWidth() ) );
     p_backgroundcolourbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, separatorfield->sizePolicy().hasHeightForWidth() ) );
 
@@ -173,6 +213,44 @@ hk_kdereportproperty::hk_kdereportproperty( hk_kdesimplereport* parent,  const c
     formattabLayout->addItem( spacer_4, 15, 0 );
     settxt=i18n("set");
     notsettxt=i18n("not set");
+
+    identifierfield->installEventFilter(this);
+    valuefield->installEventFilter(this);
+
+    QString info=i18n("Press <F2> for a larger editor window");
+    QToolTip::add(identifierfield,info);
+    QToolTip::add(valuefield,info);
+
+    QSize s=actiontab->geometry().size();
+    if (frametab->width()>s.width())s.setWidth(frametab->width());
+    if (formattab->width()>s.width())s.setWidth(formattab->width());
+    if (datatab->width()>s.width())s.setWidth(datatab->width());
+    
+    if (frametab->height()>s.height())s.setHeight(frametab->height());
+    if (formattab->height()>s.height())s.setHeight(formattab->height());
+    if (datatab->height()>s.height())s.setHeight(datatab->height());
+    
+    QSizePolicy sp=QSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
+    frametab->setSizePolicy(sp);
+    formattab->setSizePolicy(sp);
+    actiontab->setSizePolicy(sp);
+    datatab->setSizePolicy(sp);
+    
+    frametab->setMinimumSize(s);
+    formattab->setMinimumSize(s);
+    actiontab->setMinimumSize(s);
+    datatab->setMinimumSize(s);
+
+
+
+
+    tab->changeTab(p_formatscrollview,i18n("&Format"));
+    tab->changeTab(p_datascrollview,i18n("&Data"));
+    tab->changeTab(p_actionscrollview,i18n("&Action"));
+    tab->changeTab(p_framescrollview,i18n("F&rame"));
+
+
+
 }
 
 
@@ -364,9 +442,16 @@ void hk_kdereportproperty::set_object(hk_visible* v)
         }
 
     }
+
     identifierfield->setEnabled(false);
     
     p_visible=v;
+        if (p_visible)
+    { 
+      QString num;
+      num.setNum(p_visible->presentationnumber());
+      idfield->setText(num);
+    }  
     if (p_visible==NULL) return;
     QString head;
     head.setNum(p_visible->presentationnumber());
@@ -1335,3 +1420,36 @@ hk_visible* hk_kdereportproperty::object()
 {
 return p_visible;
 }
+
+
+bool hk_kdereportproperty::eventFilter(QObject* object,QEvent* event)
+{
+
+   QLineEdit* edit=dynamic_cast<QLineEdit*>(object);
+   
+   if (edit&&event->type()==QEvent::KeyPress)
+   {
+     QKeyEvent*ev=(QKeyEvent*)event;
+     if (ev->key()==Key_F2)
+       use_editor(edit);
+   } 
+  
+   return hk_kdereportpropertybase::eventFilter(object,event);
+}
+
+
+void hk_kdereportproperty::use_editor(QLineEdit* e)
+{
+  if (!e) return;
+    hk_kdefilterdialog* f = new hk_kdefilterdialog(hk_kdefilterdialog::editor,this,0,true);
+    
+    f->textfield->setText(e->text());
+    if (f->exec()==QDialog::Accepted)
+    {
+        e->setText(f->textfield->text());
+    };
+
+    delete f;
+
+}
+
