@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.25 $
+//$Revision: 1.28 $
 
 #include <hk_kderowselector.h>
 #include <hk_kderowselector.moc>
@@ -21,6 +21,7 @@
 #include <qdrawutil.h>
 #include <qnamespace.h>
 #include <qsizepolicy.h>
+#include <qtooltip.h>
 #include <hk_form.h>
 #include <hk_kdeform.h>
 
@@ -249,6 +250,7 @@ QWidget(wid),hk_dsrowselector(form)
     layout->addWidget(button_insert,1);
     layout->activate();
     layout->setEnabled(true);
+    setFocusProxy(label);
 
     QColor qcolor=button_gotofirst->paletteBackgroundColor();
     hk_colour colour(qcolor.red(),qcolor.green(),qcolor.red());
@@ -484,7 +486,7 @@ void hk_kderowselector::widget_specific_enable_disable(void)
     setEnabled(true);
   }
   else */
-    setFocusPolicy(StrongFocus);
+    //setFocusPolicy(ClickFocus);
 
 }
 
@@ -607,3 +609,27 @@ void    hk_kderowselector::widget_specific_foregroundcolour_changed(const hk_col
     button_store->setPaletteForegroundColor(newcolour);
 
 }
+
+
+void  hk_kderowselector::widget_specific_tooltip_changed(void)
+{
+ QToolTip::add(this,QString::fromLocal8Bit (tooltip().c_str()));
+}
+
+
+
+
+void hk_kderowselector::focusInEvent ( QFocusEvent * e)
+{
+  action_on_getfocus();
+  QWidget::focusInEvent(e);
+}
+
+void hk_kderowselector::focusOutEvent ( QFocusEvent * e)
+{
+  action_on_loosefocus();
+  QWidget::focusOutEvent(e);
+
+}
+
+
