@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,9 +8,10 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.86 $
+//$Revision: 1.108 $
 #include "hk_kdesimpleform.h"
 #include "hk_kdeform.h"
+#include "hk_kdeformpartwidget.h"
 #include "hk_kdegrid.h"
 #include "hk_kdesimplegrid.h"
 #include "hk_kdelineedit.h"
@@ -23,8 +24,10 @@
 #include "hk_kdeformfocus.h"
 #include "hk_kdeproperty.h"
 #include "hk_kdelabel.h"
+#include "hk_kdesubform.h"
 #include "hk_kdegridpart.h"
 #include "hk_kdedbdesigner.h"
+#include "../hk_kdeformpart/hk_kdeformpart.h"
 
 #include "hk_visible.h"
 #include "hk_dsvisible.h"
@@ -35,10 +38,14 @@
 #include <qfocusdata.h>
 #include <qpainter.h>
 #include <qevent.h>
+#include <qsplitter.h>
+#include <qlayout.h>
 #include <qcolor.h>
+#include <qtooltip.h>
 #include <kparts/partmanager.h>
 #include <klocale.h>
 #include <kaction.h>
+#include <kapplication.h>
 #include <qdragobject.h>
 #include <qclipboard.h>
 #include <qimage.h>
@@ -61,6 +68,7 @@ public:
 hk_kdesimpleformprivate()
    {
      while_modechange=false;
+     in_loop=false;
      p_dbdesigneraction=NULL;
      p_designer=NULL;
      p_propertydock=NULL;
@@ -72,6 +80,7 @@ hk_kdesimpleformprivate()
    }
 
 bool while_modechange;
+bool in_loop;
 hk_kdedbdesignerwindow* p_designer;
 KAction * p_dbdesigneraction;
 KDockWidget* p_propertydock;
@@ -79,7 +88,7 @@ KDockWidget* p_propertydock;
 
 
 
-hk_kdesimpleform::hk_kdesimpleform(QWidget* wid) :QWidget(wid),hk_form()
+hk_kdesimpleform::hk_kdesimpleform(QWidget* wid,const char* n,WFlags f) :QWidget(wid,n,f),hk_form()
 {
 #ifdef HK_DEBUG
     hkclassname("hk_kdesimpleform");
@@ -96,10 +105,11 @@ hk_kdesimpleform::hk_kdesimpleform(QWidget* wid) :QWidget(wid),hk_form()
     p_property=NULL;
     p_focus = new hk_kdeformfocus(this);
     p_createflag=false;
-    p_controlbutton=false;
+    //p_controlbutton=false;
     setFocusPolicy(StrongFocus);
     setFocus();
     p_kdeform=NULL;
+    p_formpartwidget=NULL;
     p_popup=NULL;
     p_sourcevanishflag=false;
     p_deleteaction=NULL;
@@ -135,19 +145,21 @@ hk_kdesimpleform::~hk_kdesimpleform(void)
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimpleform::~hk_kdesimpleform");
 #endif
+    set_actions(NULL);
     delete p_focus;
     //delete p_property;
-    
     while(p_partsmanager->activePart())
     {
        p_partsmanager->removePart(p_partsmanager->activePart());
     }
-    
+        if(p_property!=NULL &&!p_private->p_propertydock)
+	   delete p_property;
     delete p_partsmanager;
     delete p_deleteaction;
     delete p_copyaction;
     delete p_pasteaction;
     delete p_cutaction;
+    delete p_private->p_dbdesigneraction;
     delete p_private;
     p_private=NULL;
 }
@@ -176,7 +188,7 @@ hk_dsgrid*  hk_kdesimpleform::widget_specific_new_grid(void)
 #endif
 //	set_focus(g);
     connect(g->simplegrid(),SIGNAL(signal_infocus(hk_kdegridpart*)),this,SLOT(grid_partinfocus(hk_kdegridpart*)));
-    connect (g->simplegrid(), SIGNAL(signal_outfocus(hk_kdegridpart*)),this,SLOT(grid_partoutfocus(hk_kdegridpart*)));
+    connect (g->simplegrid(), SIGNAL(signal_outfocus(hk_kdegridpart*,QFocusEvent*)),this,SLOT(grid_partoutfocus(hk_kdegridpart*,QFocusEvent*)));
 
     return g;
 }
@@ -271,6 +283,20 @@ hk_dsrowselector*   hk_kdesimpleform::widget_specific_new_rowselector(void)
 }
 
 
+hk_subform*   hk_kdesimpleform::widget_specific_new_subform(void)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_kdesimpleform::widget_specific_new_subform(void)");
+#endif
+    hk_kdesubform* s=new hk_kdesubform(this);
+    s->show();
+#ifdef HK_DEBUG
+    s->hkclassname("subform");
+#endif
+    return s;
+}
+
+
 hk_label* hk_kdesimpleform::widget_specific_new_label(void)
 {
 #ifdef HK_DEBUG
@@ -314,6 +340,8 @@ void hk_kdesimpleform::before_source_vanishes(void)
 #endif
     bool doemit=false;
     if (p_kdeform) p_kdeform->save_form_when_changed();
+    else
+    if (p_formpartwidget)p_formpartwidget->save_form_when_changed();
     hk_form::before_source_vanishes();
     
    QWidget* p=parentWidget();
@@ -335,6 +363,12 @@ if (doemit)
        set_kdeform(NULL);
        k->close();
       } 
+    else
+    if (p_formpartwidget)
+      {hk_kdeformpartwidget* k=p_formpartwidget;
+       set_formpartwidget(NULL);
+       k->close();
+      } 
     else p->close();
    close();
 }
@@ -366,6 +400,12 @@ if (doemit)
        set_kdeform(NULL);
        k->close();
       } 
+    else 
+        if (p_formpartwidget)
+      {hk_kdeformpartwidget* k=p_formpartwidget;
+       set_formpartwidget(NULL);
+       k->close();
+      } 
     else p->close();
    close();
 }
@@ -410,6 +450,16 @@ void    hk_kdesimpleform::mousePressEvent(QMouseEvent* m)
 }
 
 
+void  hk_kdesimpleform::mouseDoubleClickEvent(QMouseEvent* event)
+{
+   if (mode()==viewmode
+       && event->button()== LeftButton)
+           action_on_doubleclick();
+   QWidget::mouseDoubleClickEvent(event);
+}
+
+
+
 void    hk_kdesimpleform::mouseReleaseEvent(QMouseEvent* )
 {
 }
@@ -429,10 +479,10 @@ void hk_kdesimpleform::create_field(const QPoint& position)
 #endif
     hk_visible* newfield=NULL;
     QPoint point=position;
-	if (snap2gridx()>0&&point.x()%snap2gridx()>0&&!p_controlbutton)
-		point.setX(point.x()-point.x()%snap2gridx());
-	if (snap2gridy()>0&&point.y()%snap2gridy()>0&&!p_controlbutton)
-		point.setY(point.y()-point.y()%snap2gridy());
+	if (snap2gridx()>0&&point.x()%snap2gridx()>0)
+		point.setX((point.x()/snap2gridx())*snap2gridx());
+	if (snap2gridy()>0&&point.y()%snap2gridy()>0)
+		point.setY((point.y()/snap2gridy())*snap2gridy());
 
     switch (p_field2create)
     {
@@ -452,6 +502,8 @@ void hk_kdesimpleform::create_field(const QPoint& position)
         break;
         case combobox : newfield = new_combobox();
         break;
+        case subform : newfield = new_subform();
+        break;
         default:;
 
     }
@@ -471,7 +523,7 @@ void hk_kdesimpleform::create_field(const QPoint& position)
 		newfield->set_size((unsigned int)point.x(),(unsigned int)point.y(),100,30);
 
 	}
-        set_focus(dynamic_cast <QWidget*>(newfield));
+        set_focus(dynamic_cast <QWidget*>(newfield),false);
     }
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimpleform::create_field focus setzen");
@@ -498,7 +550,7 @@ void hk_kdesimpleform::paintEvent(QPaintEvent* e)
     for (int a=0; a<QWidget::width()/10+20; a++ )
     {
         for (int b=0; b<QWidget::height()/10+20;b++ )
-            p.drawPoint(a*10,b*10);
+            p.drawPoint(a*10,b*10+3);
     }
 
 }
@@ -534,7 +586,9 @@ bool hk_kdesimpleform::formeventFilter(QObject* object,QEvent* event)
         QWidget* object =widget;
         while (object!=NULL)
         {
-            if (dynamic_cast <hk_kdegrid*>(object)!=NULL)
+            if ( dynamic_cast <hk_kdegrid*>(object)!=NULL
+	       ||dynamic_cast <hk_kdesubform*>(object)!=NULL
+	       )
             {
                 widget=object;
                 object=NULL;
@@ -556,6 +610,7 @@ bool hk_kdesimpleform::formeventFilter(QObject* object,QEvent* event)
         }
     }
 
+    
     if (dynamic_cast <hk_kderowselector*>(widget)!=NULL)
     {
         if (dynamic_cast <hk_kdegrid*>(widget->parentWidget())!=NULL)
@@ -594,11 +649,13 @@ bool hk_kdesimpleform::formeventFilter(QObject* object,QEvent* event)
                 if ( dynamic_cast <QPopupMenu*>(object))
                     return QWidget::eventFilter(object,event);
 
-                set_focus(widget);
+                set_focus(widget,((QMouseEvent*)event)->state() &ControlButton);
                 if (widget!=NULL)p_focus->show();
                 p_originalposition=mapFromGlobal(((QMouseEvent*)event)->globalPos());
-		if (snap2gridx()>0&&p_originalposition.x()%snap2gridx()>0&&!p_controlbutton)p_originalposition.setX(p_originalposition.x()-p_originalposition.x()%snap2gridx());
-		if (snap2gridy()>0&&p_originalposition.y()%snap2gridy()>0&&!p_controlbutton)p_originalposition.setY(p_originalposition.y()-p_originalposition.y()%snap2gridy());
+		if (snap2gridx()>0 &&(!(((QMouseEvent*)event)->state()&Qt::ControlButton)))
+		  p_originalposition.setX((p_originalposition.x()/snap2gridx())*snap2gridx());
+		if (snap2gridy()>0&&(!(((QMouseEvent*)event)->state()&Qt::ControlButton)))
+		  p_originalposition.setY((p_originalposition.y()/snap2gridy())*snap2gridy());
                 return true;
             }
             else
@@ -618,10 +675,10 @@ bool hk_kdesimpleform::formeventFilter(QObject* object,QEvent* event)
             {
 //move object
                 QPoint newpos=mapFromGlobal(((QMouseEvent*)event)->globalPos());
-		if (snap2gridx()>0&&newpos.x()%snap2gridx()>0&&!p_controlbutton)
-			newpos.setX(newpos.x()-newpos.x()%snap2gridx());
-		if (snap2gridy()>0&&newpos.y()%snap2gridy()>0&&!p_controlbutton)
-			newpos.setY(newpos.y()-newpos.y()%snap2gridy());
+		if (snap2gridx()>0&&!(((QMouseEvent*)event)->state() & ControlButton))
+			newpos.setX((newpos.x()/snap2gridx())*snap2gridx());
+		if (snap2gridy()>0&&!(((QMouseEvent*)event)->state() & ControlButton))
+			newpos.setY((newpos.y()/snap2gridy())*snap2gridy());
 		QPoint difference=p_originalposition-newpos;
 		QPoint buf=p_originalposition;
                 p_originalposition=newpos;
@@ -669,9 +726,12 @@ void hk_kdesimpleform::widget_specific_modechanges(enum_mode s)
         setFocus();
         qApp->installEventFilter(this);
         show_property();
+	if (p_property)
+	{
         if (p_focus->widget()!=NULL)
             p_property->set_object( dynamic_cast <hk_visible*> (p_focus->widget()));
         else p_property->set_object(this);
+	}
         while (it!=p_multiplefocus.end())
         {
             hk_kdeformfocus* f=(*it);
@@ -703,7 +763,24 @@ void hk_kdesimpleform::widget_specific_modechanges(enum_mode s)
 	       p_property->hide();
 	  }
         setFocusPolicy(ClickFocus);
-
+	if (!is_subform())
+	{
+	QWidget* w=dynamic_cast<QWidget*> (first_tabobject());
+	if (w)
+	{
+	 w->setFocus();
+	 QLineEdit* e=dynamic_cast<QLineEdit*>(w);
+	 if (e) e->selectAll();
+	 else
+	 {
+	   QTextEdit* t=dynamic_cast<QTextEdit*>(w);
+	   if (t) t->selectAll();
+	 }
+	 //cerr <<"FOCUS SETZEN"<<w->className()<<endl;
+	} 
+	else
+	cerr <<" NO FOCUS"<<endl;
+        }
     }
     enable_actions();
 }
@@ -739,61 +816,63 @@ void hk_kdesimpleform::keyPressEvent ( QKeyEvent * event )
 #endif
     switch(event->key())
     {
-        case Key_Control:  p_controlbutton=true;
-        break;
    case Key_Left:
             if (p_focus &&p_focus->widget()&&mode()==designmode)
             {
               hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
               if (widget)
               {
-               unsigned int diff=(p_controlbutton?1:snap2gridx());
+               unsigned int diff=(event->state()&ControlButton?1:snap2gridx());
 	       if (sizetype()==relative) diff =horizontal2relativ(diff);
                if (widget->x()>=diff) widget->set_x(widget->x()-diff);
               }
 
             }
-break;
-case Key_Right:
-if (p_focus &&p_focus->widget()&&mode()==designmode)
-{
-hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
-if (widget)
-{
-   unsigned int diff=(p_controlbutton?1:snap2gridx());
+	break;
+    
+    case Key_Right:
+	if (p_focus &&p_focus->widget()&&mode()==designmode)
+	{
+	hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
+	if (widget)
+	{
+   	unsigned int diff=(event->state()&ControlButton?1:snap2gridx());
 	       if (sizetype()==relative) diff =horizontal2relativ(diff);
-if (widget->x()+widget->width()+diff<=10000) widget->set_x(widget->x()+diff);
-}
+	if (widget->x()+widget->width()+diff<=10000) widget->set_x(widget->x()+diff);
+	}
 
-}
-break;
-case Key_Up:
-if (p_focus &&p_focus->widget()&&mode()==designmode)
-{
-hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
-if (widget)
-{
-unsigned int diff=(p_controlbutton?1:snap2gridy());
+	}
+	break;
+
+     case Key_Up:
+	if (p_focus &&p_focus->widget()&&mode()==designmode)
+	{
+	hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
+	if (widget)
+	{
+	unsigned int diff=(event->state()&ControlButton?1:snap2gridy());
 	       if (sizetype()==relative) diff =vertical2relativ(diff);
-if (widget->y()>=diff) widget->set_y(widget->y()-diff);
-}
+	if (widget->y()>=diff) widget->set_y(widget->y()-diff);
+	}
 
-}
-break;
-case Key_Down:
-if (p_focus &&p_focus->widget()&&mode()==designmode)
-{
-hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
-if (widget)
-{
-unsigned int diff=(p_controlbutton?1:snap2gridy());
+	}
+	break;
+
+     case Key_Down:
+	if (p_focus &&p_focus->widget()&&mode()==designmode)
+	{
+	hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
+	if (widget)
+	{
+	unsigned int diff=(event->state()&ControlButton?1:snap2gridy());
 	       if (sizetype()==relative) diff =vertical2relativ(diff);
-if (widget->y()+widget->height()+diff<=10000) widget->set_y(widget->y()+diff);
-}
+	if (widget->y()+widget->height()+diff<=10000) widget->set_y(widget->y()+diff);
+	}
 
-}
-break;
-        default  :     ;
+	}
+	break;
+        
+	default  :     ;
     }
     QWidget::keyPressEvent(event);
 
@@ -805,7 +884,7 @@ void hk_kdesimpleform::keyReleaseEvent ( QKeyEvent * event)
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimpleform::keyReleaseEvent");
 #endif
-    if (event->key()==Key_Control)  p_controlbutton=false;
+//    if (event->key()==Key_Control)  p_controlbutton=false;
     QWidget::keyReleaseEvent(event);
 
 }
@@ -817,23 +896,49 @@ void hk_kdesimpleform::clearfocus(void)
     hkdebug("hk_kdesimpleform::clearfocus");
 #endif
     if (p_focus!=NULL) p_focus->set_widget(NULL);
-    p_controlbutton=false;
+    //p_controlbutton=false;
     clearmultiplefocus();
     enable_actions();
 
 }
 
 
-void hk_kdesimpleform::set_focus(QWidget*f)
+void hk_kdesimpleform::set_focus(QWidget*f, bool p_controlbutton)
 {
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimpleform::set_focus");
 #endif
+
     if (f==NULL || f==this)
     {
         clearfocus();
+	emit signal_focuswidget_changed();
         return;
     }
+    
+        QWidget* widget=dynamic_cast<hk_kdesimpleform*>(f);
+	if (widget)
+	{
+        widget = widget->parentWidget();
+        QWidget* object =widget;
+        while (object!=NULL)
+        {
+            if ( dynamic_cast <hk_kdesubform*>(object)!=NULL   
+	        ||dynamic_cast <hk_kdegrid*>(object)!=NULL
+	       )
+            {
+                widget=object;
+                object=NULL;
+            }
+            else    object=object->parentWidget();
+        }
+	if (widget) f=widget;
+        }
+    
+    
+    
+    
+    
     if (p_controlbutton==true)
     {
 
@@ -867,7 +972,8 @@ void hk_kdesimpleform::set_focus(QWidget*f)
         QObject::connect(p_focus,SIGNAL(size_changed()),p_property,SLOT(focus_resized()));
         p_property->set_object(v);
     }
-
+    
+emit signal_focuswidget_changed();
 }
 
 
@@ -1063,7 +1169,10 @@ void hk_kdesimpleform::show_property(void)
     bool newproperty=false;
     if (p_property==NULL)
     {   
-         p_property=new hk_kdeproperty(this,0,false);
+        
+         if (p_kdeform||p_formpartwidget)
+	 { 
+	   p_property=new hk_kdeproperty(this,0,false);
          if (p_kdeform)
         {
 	 QPixmap p;
@@ -1071,10 +1180,24 @@ void hk_kdesimpleform::show_property(void)
 	 p_private->p_propertydock->setWidget(p_property);
 	 p_private->p_propertydock->manualDock(p_kdeform->getMainDockWidget(),KDockWidget::DockRight);
 	}
-       newproperty=true;
+	else
+	if (p_formpartwidget)
+        {
+	  /* p_property->reparent(p_formpartwidget->p_splitter, 0, QPoint(0,0), false);
+	   QVBoxLayout* layout=new QVBoxLayout(p_formpartwidget->p_splitter);
+	   layout->addWidget(p_property);*/
 
+	 
+	 QPixmap p;
+	 p_private->p_propertydock=p_formpartwidget->createDockWidget("Property editor",p,0L,"");
+	 p_private->p_propertydock->setWidget(p_property);
+	 p_private->p_propertydock->manualDock(p_formpartwidget->getMainDockWidget(),KDockWidget::DockRight);
+	}
+
+       newproperty=true;
+       }
     }
-    if (p_property!=NULL)
+    if (p_property!=NULL &&!masterform())
     {   
      if (p_private->p_propertydock)
         {
@@ -1210,6 +1333,8 @@ void hk_kdesimpleform::widget_specific_presentationdatasource(long p)
     hkdebug("hk_kdesimpleform::widget_specific_new_presentationdatasource");
 #endif
     if (p_kdeform!=NULL) p_kdeform->set_presentationdatasource(p,false);
+    else
+    if (p_formpartwidget!=NULL) p_formpartwidget->set_presentationdatasource(p,false);
 
 }
 
@@ -1220,23 +1345,38 @@ void hk_kdesimpleform::set_kdeform(hk_kdeform* k)
     hkdebug("hk_kdesimpleform::set_kdeform");
 #endif
     p_kdeform=k;
-    if (p_kdeform)
+    set_actions(p_kdeform?p_kdeform->actionCollection():NULL);
+}
+
+void hk_kdesimpleform::set_formpartwidget(hk_kdeformpartwidget* k)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_kdesimpleform::set_kdeformpartwidget");
+#endif
+    p_formpartwidget=k;
+ //   set_actions(p_formpartwidget?p_formpartwidget->formpart()->actionCollection():NULL);
+}
+
+
+void hk_kdesimpleform::set_actions(KActionCollection* collection)
+{
+   if (collection)
     {
     KIconLoader* loader=KGlobal::iconLoader();
     loader->addAppDir("hk_kdeclasses");
-    p_deleteaction=new KAction(i18n("&Delete"),Key_Delete,this,SLOT(delete_widgets()),p_kdeform->actionCollection(),"deleteclicked");
-    p_copyaction=new KAction(i18n("&Copy"),CTRL+Key_C,this,SLOT(copy()),p_kdeform->actionCollection(),"copy");
-    p_pasteaction=new KAction(i18n("&Paste"),CTRL+Key_V,this,SLOT(paste()),p_kdeform->actionCollection(),"paste");
-    p_cutaction=new KAction(i18n("Cu&t"),CTRL+Key_X,this,SLOT(cut()),p_kdeform->actionCollection(),"cutclicked");
-    p_formpropertyaction=new KAction(i18n("&Propertyeditor"),loader->loadIcon("propertyeditor",KIcon::User),0,this,SLOT(show_property()),p_kdeform->actionCollection(),"viewproperty");
+    p_deleteaction=new KAction(i18n("&Delete"),Key_Delete,this,SLOT(delete_widgets()),collection,"deleteclicked");
+    p_copyaction=new KAction(i18n("&Copy"),CTRL+Key_C,this,SLOT(copy()),collection,"copy");
+    p_pasteaction=new KAction(i18n("&Paste"),CTRL+Key_V,this,SLOT(paste()),collection,"paste");
+    p_cutaction=new KAction(i18n("Cu&t"),CTRL+Key_X,this,SLOT(cut()),collection,"cutclicked");
+    p_formpropertyaction=new KAction(i18n("&Propertyeditor"),loader->loadIcon("propertyeditor",KIcon::User),0,this,SLOT(show_property()),collection,"viewproperty");
 
-    p_resizeaction=new KActionMenu(i18n("Adjust &size"),0,p_kdeform->actionCollection(),"size");
-    p_minwidthaction=new KAction(i18n("Minimum width"),0,0,this,SLOT(adjust_minw()),p_kdeform->actionCollection(),"minwidth");
-    p_maxwidthaction=new KAction(i18n("Maximum width"),0,0,this,SLOT(adjust_maxw()),p_kdeform->actionCollection(),"maxwidth");
-    p_minheightaction=new KAction(i18n("Minimum height"),0,0,this,SLOT(adjust_minh()),p_kdeform->actionCollection(),"minheight");
-    p_maxheightaction=new KAction(i18n("Maximum height"),0,0,this,SLOT(adjust_maxh()),p_kdeform->actionCollection(),"maxheight");
-    p_minsizeaction=new KAction(i18n("Minimum size"),0,0,this,SLOT(adjust_mins()),p_kdeform->actionCollection(),"minsize");
-    p_maxsizeaction=new KAction(i18n("Maximum size"),0,0,this,SLOT(adjust_maxs()),p_kdeform->actionCollection(),"maxsize");
+    p_resizeaction=new KActionMenu(i18n("Adjust &size"),0,collection,"size");
+    p_minwidthaction=new KAction(i18n("Minimum width"),0,0,this,SLOT(adjust_minw()),collection,"minwidth");
+    p_maxwidthaction=new KAction(i18n("Maximum width"),0,0,this,SLOT(adjust_maxw()),collection,"maxwidth");
+    p_minheightaction=new KAction(i18n("Minimum height"),0,0,this,SLOT(adjust_minh()),collection,"minheight");
+    p_maxheightaction=new KAction(i18n("Maximum height"),0,0,this,SLOT(adjust_maxh()),collection,"maxheight");
+    p_minsizeaction=new KAction(i18n("Minimum size"),0,0,this,SLOT(adjust_mins()),collection,"minsize");
+    p_maxsizeaction=new KAction(i18n("Maximum size"),0,0,this,SLOT(adjust_maxs()),collection,"maxsize");
     p_resizeaction->insert(p_minwidthaction);
     p_resizeaction->insert(p_maxwidthaction);
     p_resizeaction->insert(p_minheightaction);
@@ -1244,19 +1384,39 @@ void hk_kdesimpleform::set_kdeform(hk_kdeform* k)
     p_resizeaction->insert(p_minsizeaction);
     p_resizeaction->insert(p_maxsizeaction);
 
-    p_alignaction=new KActionMenu(i18n("&Align"),0,p_kdeform->actionCollection(),"align");
-    p_alignleftaction=new KAction(i18n("&Left"),0,0,this,SLOT(align_left()),p_kdeform->actionCollection(),"alignleft");
-    p_alignrightaction=new KAction(i18n("&Right"),0,0,this,SLOT(align_right()),p_kdeform->actionCollection(),"alignright");
-    p_aligntopaction=new KAction(i18n("&Top"),0,0,this,SLOT(align_top()),p_kdeform->actionCollection(),"aligntop");
-    p_alignbottomaction=new KAction(i18n("&Bottom"),0,0,this,SLOT(align_bottom()),p_kdeform->actionCollection(),"alignbottom");
+    p_alignaction=new KActionMenu(i18n("&Align"),0,collection,"align");
+    p_alignleftaction=new KAction(i18n("&Left"),0,0,this,SLOT(align_left()),collection,"alignleft");
+    p_alignrightaction=new KAction(i18n("&Right"),0,0,this,SLOT(align_right()),collection,"alignright");
+    p_aligntopaction=new KAction(i18n("&Top"),0,0,this,SLOT(align_top()),collection,"aligntop");
+    p_alignbottomaction=new KAction(i18n("&Bottom"),0,0,this,SLOT(align_bottom()),collection,"alignbottom");
     p_alignaction->insert(p_alignleftaction);
     p_alignaction->insert(p_alignrightaction);
     p_alignaction->insert(p_aligntopaction);
     p_alignaction->insert(p_alignbottomaction);
-    p_private->p_dbdesigneraction=new KAction(i18n("Database designer"),loader->loadIcon("dbdesigner",KIcon::User),0,this,SLOT(dbdesignaction()),p_kdeform->actionCollection(),"dbdesigner");
+    p_private->p_dbdesigneraction=new KAction(i18n("Database designer"),loader->loadIcon("dbdesigner",KIcon::User),0,this,SLOT(dbdesignaction()),collection,"dbdesigner");
    }
    else
    {
+    if (p_deleteaction)p_deleteaction->unplugAll();
+    if (p_copyaction)p_copyaction->unplugAll();
+    if (p_pasteaction)p_pasteaction->unplugAll();
+    if (p_cutaction)p_cutaction->unplugAll();
+    if (p_formpropertyaction)p_formpropertyaction->unplugAll();
+
+    if (p_resizeaction)p_resizeaction->unplugAll();
+    if (p_minwidthaction)p_minwidthaction->unplugAll();
+    if (p_maxwidthaction) p_maxwidthaction->unplugAll();
+    if (p_minheightaction) p_minheightaction->unplugAll();
+    if (p_maxheightaction)p_maxheightaction->unplugAll();
+    if (p_minsizeaction) p_minsizeaction->unplugAll();
+    if (p_maxsizeaction) p_maxsizeaction->unplugAll();
+    if (p_alignaction) p_alignaction->unplugAll();
+    if (p_alignleftaction) p_alignleftaction->unplugAll();
+    if (p_alignrightaction) p_alignrightaction->unplugAll();
+    if (p_aligntopaction) p_aligntopaction->unplugAll();
+    if (p_alignbottomaction) p_alignbottomaction->unplugAll();
+    if (p_private->p_dbdesigneraction)
+       p_private->p_dbdesigneraction->unplugAll();
     delete p_deleteaction;
     delete p_copyaction;
     delete p_pasteaction;
@@ -1276,7 +1436,6 @@ void hk_kdesimpleform::set_kdeform(hk_kdeform* k)
     delete p_aligntopaction;
     delete p_alignbottomaction;
     delete p_private->p_dbdesigneraction;
-
     
     p_deleteaction=NULL;
     p_copyaction=NULL;
@@ -1299,9 +1458,13 @@ void hk_kdesimpleform::set_kdeform(hk_kdeform* k)
     p_private->p_dbdesigneraction=NULL;
    
    }
+    //if (p_property) delete p_property;
+    //p_property=NULL;
 }
 
 
+
+
 bool hk_kdesimpleform::is_formobject(QWidget* v)
 {
 #ifdef HK_DEBUG
@@ -1384,7 +1547,7 @@ void hk_kdesimpleform::paste_widgets(void)
     if (!QApplication::clipboard()->data()) return;
     if (!QApplication::clipboard()->data()->encodedData("application/x-hk_kdesimpleformcopy"))return ;
     clearfocus();
-    p_controlbutton=true;
+    //p_controlbutton=true;
     hk_string definition=u2l(QApplication::clipboard()->data()->encodedData("application/x-hk_kdesimpleformcopy").data());
     cout <<"einzufgen=:"<<endl<<definition<<endl<<"ENDE"<<endl;
     int i=1;
@@ -1400,7 +1563,7 @@ void hk_kdesimpleform::paste_widgets(void)
             vis->loaddata(value);
             register_object(vis);
             QWidget* w=dynamic_cast<QWidget*>(vis);
-            if (w)set_focus(w);
+            if (w)set_focus(w,true);
         }
         i++;
     }
@@ -1450,20 +1613,22 @@ bool    hk_kdesimpleform::set_mode(enum_mode s)
 
    if (s==hk_presentation::viewmode)
     {
-        p_formpropertyaction->setEnabled(false);
+        if (p_formpropertyaction)p_formpropertyaction->setEnabled(false);
 	if (p_private->p_designer)
 		{
 			delete p_private->p_designer;
 			p_private->p_designer=NULL;
 		}
-
+     repaint();
     }
     else
     {
-        p_formpropertyaction->setEnabled(true);
+        if(p_formpropertyaction)p_formpropertyaction->setEnabled(true);
 
     }
-    p_kdeform->set_mode(s);
+    if (p_kdeform)p_kdeform->set_mode(s);
+    else
+    if (p_formpartwidget)p_formpartwidget->set_mode(s);
     p_private->while_modechange=false;
 
     return r;
@@ -1592,11 +1757,16 @@ hk_kdeproperty* hk_kdesimpleform::propertyeditor(void)
 }
 
 
-hk_kdeform* hk_kdesimpleform::kdeform()
+hk_kdeform* hk_kdesimpleform::kdeform() const
 {
    return p_kdeform;
 }
 
+hk_kdeformpartwidget* hk_kdesimpleform::formpartwidget(void) const
+{
+
+return  p_formpartwidget;
+}
 
 void hk_kdesimpleform::script_error(hk_visible* v, hk_interpreter::enum_action a)
 {
@@ -1612,6 +1782,14 @@ void hk_kdesimpleform::script_error(hk_visible* v, hk_interpreter::enum_action a
     hk_string error=replace_all("%LINENUMBER%",hk_translate("Line %LINENUMBER%: "),longint2string(interpreter()->error_rownumber()))
     				+interpreter()->errormessage();
 
+hk_dsgridcolumn* gc=dynamic_cast<hk_dsgridcolumn*>(v);
+if (gc)
+  {
+    error=replace_all("%1",hk_translate("Gridcolumn '%1':\n"),gc->columnname())+error;
+  
+  
+  }
+
 hk_dsvisible* dv=dynamic_cast<hk_dsvisible*>(v);
 if (dv)
    {
@@ -1621,10 +1799,21 @@ if (dv)
 
 if (!runtime_only())
    {
-      kdeform()->set_mode(hk_presentation::designmode);
-   	 if (p_property)
+      
+     if (!gc||(gc &&!a==hk_interpreter::a_on_key))
+     { 
+      if (kdeform()) kdeform()->set_mode(hk_presentation::designmode);
+        else set_mode(hk_presentation::designmode);
+     }
+     else
+     {
+       show_warningmessage(error);
+       return;
+     }
+    if (p_property)
 	    {
-	     set_focus(dynamic_cast<QWidget*>(v));
+	     set_focus(dynamic_cast<QWidget*>(v),false);
+	     p_property->set_object(v);
 	     switch (a)
 	     {
 		case hk_interpreter::a_click :
@@ -1633,12 +1822,24 @@ if (!runtime_only())
 		case hk_interpreter::a_doubleclick :
 				p_property->doubleclickactionbutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
 				break;
-		case hk_interpreter::a_onopen :
+		case hk_interpreter::a_on_open :
 				p_property->openactionbutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
 				break;
-		case hk_interpreter::a_onclose :
+		case hk_interpreter::a_on_close :
 				p_property->closeactionbutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
 				break;
+		
+		case hk_interpreter::a_on_getfocus :
+				p_property->ongetfocusbutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
+				break;
+		case hk_interpreter::a_on_loosefocus :
+				p_property->onloosefocusbutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
+				break;
+		
+		case hk_interpreter::a_on_key :
+				p_property->onkeybutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
+				break;
+		
 		case hk_interpreter::a_after_row_change :
 				p_property->afterrowchangebutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
 				break;
@@ -1666,7 +1867,7 @@ if (!runtime_only())
 			default:;
 
 	     }
-           }
+           }else show_warningmessage("No Property editor");
 
    }
    else     show_warningmessage(error);
@@ -1676,22 +1877,38 @@ if (!runtime_only())
 }
 
 
-bool hk_kdesimpleform::ctrl_key_pressed(void) const
+/*bool hk_kdesimpleform::ctrl_key_pressed(void) const
 {
 return p_controlbutton;
-}
+}*/
 
 
 void hk_kdesimpleform::grid_partinfocus(hk_kdegridpart* p)
 {
-if (!p_kdeform) return;
-p_kdeform->grid_partinfocus(p);
-if (!p) return;
+if (p_kdeform) p_kdeform->grid_partinfocus(p);
+else
+if (p_formpartwidget) p_formpartwidget->grid_partinfocus(p);
+
 }
 
-void hk_kdesimpleform::grid_partoutfocus(hk_kdegridpart*)
+void hk_kdesimpleform::grid_partoutfocus(hk_kdegridpart* g,QFocusEvent* e)
 {
+if (g)
+{
+  hk_kdegrid* grid=g->grid();
+  if (e->reason()==QFocusEvent::Tab || e->reason()==QFocusEvent::Backtab )
+  {
+  QWidget* w=dynamic_cast<QWidget*>(e->reason()==QFocusEvent::Backtab
+  		?taborder_previous(grid)
+	      	:taborder_next(grid));
+  
+  if (w) w->setFocus();
+  }
+}
+
 if (p_kdeform)p_kdeform->grid_partoutfocus(0L);
+else
+if (p_formpartwidget)p_formpartwidget->grid_partoutfocus(0L);
 /*p_copyaction->setEnabled(false);
 p_pasteaction->setEnabled(false);*/
 
@@ -1793,4 +2010,214 @@ void hk_kdesimpleform::designer_deleted(void)
 p_private->p_designer=NULL;
 }
 
+bool hk_kdesimpleform::focus_multipleselected(void) const
+{
+return p_multiplefocus.size()>0;
+}
+
+
+void hk_kdesimpleform::set_has_changed(enum_has_changed forcesetting)
+{
+hk_form::set_has_changed(forcesetting);
+emit signal_has_changed();
+}
+
+
+
+void hk_kdesimpleform::reset_has_changed(void)
+{
+  hk_form::reset_has_changed();
+emit signal_has_changed();
+}
+
+
+void hk_kdesimpleform::load_form(const hk_string& name)
+{
+hk_form::load_form(name);
+if (p_formpartwidget) p_formpartwidget->set_caption();
+}
+
+
+
+bool hk_kdesimpleform::focusNextPrevChild(bool forward)
+{
+  hk_visible* v=NULL;
+  QWidget* wid=focusData()->home();
+  while(!v && wid)
+  {
+  v=dynamic_cast<hk_visible*>(wid);
+  hk_kdesimplegrid* grid=dynamic_cast<hk_kdesimplegrid*>(wid);
+  if (grid) 
+  {
+   v=grid->kdegrid();
+  } 
+  else
+  wid=wid->parentWidget();
+  }
+  if (!v) 
+   {
+     cerr <<"hk_kdesimpleform::focusNextPrevChild focusData kein hk_visible objekt!"<<endl;
+     return false;
+   }
+  if (forward)
+    v=taborder_next(v);
+  else
+    v=taborder_previous(v);
+  
+  if (!v) 
+   {
+     cerr <<"hk_kdesimpleform::focusNextPrevChild taborder_next/previsous gab NULL zurck!"<<endl;
+     return false;
+   }
+  QWidget* w=dynamic_cast<QWidget*>(v);
+  if (w)
+   {
+   w->setFocus(); 
+   }
+  return true;
+}
+
+
+void hk_kdesimpleform::goto_taborder_next()
+{
+focusNextPrevChild(true);
+}
+
+void hk_kdesimpleform::goto_taborder_previous()
+{
+focusNextPrevChild(false);
+}
+
+void hk_kdesimpleform::goto_taborder_first()
+{
+  QWidget* w=dynamic_cast<QWidget*>(first_tabobject());
+  if (w)
+   {
+   w->setFocus(); 
+   }
+
+}
+
+void hk_kdesimpleform::goto_taborder_last()
+{
+  QWidget* w=dynamic_cast<QWidget*>(last_tabobject());
+  if (w)
+   {
+   w->setFocus(); 
+   }
+
+
+}
+
+void hk_kdesimpleform::set_focus(hk_visible* v)
+{
+  if (!v) return;
+  QWidget* w=dynamic_cast<QWidget*>(v);
+  if (w) w->setFocus();
+
+}
+
+
+
+void  hk_kdesimpleform::widget_specific_tooltip_changed(void)
+{
+ QToolTip::add(this,QString::fromLocal8Bit (tooltip().c_str()));
+}
+
+
+
+
+void hk_kdesimpleform::focusInEvent ( QFocusEvent * e)
+{
+  action_on_getfocus();
+  QWidget::focusInEvent(e);
+}
+
+void hk_kdesimpleform::focusOutEvent ( QFocusEvent * e)
+{
+  action_on_loosefocus();
+  QWidget::focusOutEvent(e);
+
+}
+
+
+
+
+
+bool hk_kdesimpleform::show_asdialog(void)
+{
+    if ( p_private->in_loop ) {
+	qWarning( "hk_kdesimpleform::exec: Recursive call detected." );
+	return false;
+    }
+
+    //bool destructiveClose = testWFlags( WDestructiveClose );
+    clearWFlags( WDestructiveClose );
+
+    bool wasShowModal = testWFlags( WShowModal );
+    setWFlags( WShowModal| WType_Dialog );
+
+    show();
+    p_private->in_loop = TRUE;
+    qApp->enter_loop();
+
+    if ( !wasShowModal )
+	clearWFlags( WShowModal );
+return true;
+
+}
+
+
+void hk_kdesimpleform::done( void )
+{
+    list<hk_datasource*>* l=datasources();
+    list<hk_datasource*>::iterator it=l->begin();
+    bool store_ok=true;
+    while(it!=l->end()&& store_ok)
+    {
+    if ((*it)->has_changed()) 
+      store_ok=(*it)->store_changed_data();
+    ++it;
+    }
+    
+    if (store_ok)hide();
+    close();
+
+}
+
+void hk_kdesimpleform::closeEvent ( QCloseEvent* e)
+{
+ if (p_private->in_loop)
+   done();
+ if (isHidden())e->accept();
+ else e->ignore();
+}
+
+
+void hk_kdesimpleform::hide()
+{
+    if ( isHidden() )
+	return;
+
+
+    QWidget::hide();
+    if ( p_private->in_loop ) {
+	p_private->in_loop = false;
+	qApp->exit_loop();
+    }
+}
+
+
+bool hk_kdesimpleform::widget_specific_coordinates(uint px,uint py,uint ,uint )
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_kdesimpleform::widget_specific_coordinates");
+#endif
+    if (p_kdeform ||p_formpartwidget) return true;
+    move(px,py);
+    qApp->processEvents();
+
+    return true;
+}
+
 
