@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.95 $
+//$Revision: 1.117 $
 
 #include <hk_kdesimplegrid.h>
 #include <hk_kdesimplegrid.moc>
@@ -29,6 +29,7 @@
 #include <qpopupmenu.h>
 #include <qiconset.h>
 #include <qlistbox.h>
+#include <qtooltip.h>
 #include "hk_kdesimpleform.h"
 #include "hk_kdegrid.h"
 #include "hk_kdegridcolumndialog.h"
@@ -164,25 +165,56 @@ void internalcheckbox::paintEvent(QPaintEvent*)
     int top=(contentsRect().height()+1-square)/2 ;
     painter.fillRect( 0, 0, QWidget::width(), QWidget::height(),  farbe.brush( QColorGroup::Base ) );
     if (p_cursoron) qDrawShadeRect(&painter,left,top,square,square,farbe,false,1);
-    bool draw=false;
+    int draw=0;
     if (column()!=NULL)
     {
         if (column()->has_changed())
         {
-            draw=column()->changed_data_asbool();
+            if (column()->changed_data_is_nullvalue()) draw=2;
+	    else
+            if (column()->changed_data_asbool()) draw=1;
+	    
         }
         else
         {
-            draw=column()->asbool();
+            if (column()->is_nullvalue()) draw=2;
+	    else
+	    if(column()->asbool()) draw=1;
         }
     }
-    if (draw)
 
+
+
+
+
+
+    switch (draw)
     {
+    case 1: { // TRUE
 
-        const QBrush brush =QBrush(colorGroup().dark());
+        const QBrush brush =QBrush(colorGroup().highlightedText());
         qDrawPlainRect(&painter,left+3,top+3,square/2,square/2,colorGroup().dark(),1,&brush);
-    }
+            break;
+	    }
+   case 2:	 {// NULL Value
+         QColor nullcolor=colorGroup().text();
+        QBrush brush =QBrush(nullcolor);
+	brush.setStyle(QBrush::Dense5Pattern);
+        qDrawPlainRect(&painter,left+2,top+2,square-4,square-4,nullcolor,0,&brush);
+	    
+	    
+	    
+	    break;
+	    }
+   default:;	        
+   }
+
+
+
+
+
+
+
 }
 
 
@@ -212,9 +244,30 @@ void internalcheckbox::mousePressEvent(QMouseEvent* )
     if (column()!=NULL)
     {
         if (column()->has_changed())
-            column()->set_asbool(!column()->changed_data_asbool());
+	{
+           if (column()->changed_data_is_nullvalue())column()->set_asbool(true);
+	   else
+	   if ( column()->changed_data_asbool()) column()->set_asbool(false);
+	   else
+	   {
+	    if (column()->is_notnull())column()->set_asbool(true);
+	    else
+	    column()->set_asnullvalue();
+	   } 
+	}   
         else
-            column()->set_asbool(!column()->asbool());
+	{
+           if (column()->is_nullvalue())column()->set_asbool(true);
+	   else
+	   if ( column()->asbool()) column()->set_asbool(false);
+	   else
+	   {
+	    if (column()->is_notnull())column()->set_asbool(true);
+	    else
+	    column()->set_asnullvalue();
+	   } 
+//            column()->set_asbool(!column()->asbool());
+	}    
     }else cerr <<"internalcheckbox::mousePressEvent() ist NULL"<<endl;
     blinkcursorslot();
 }
@@ -254,6 +307,7 @@ hk_dsvisible(form)
     loader->addAppDir("hk_kdeclasses");
     p_grid=grid;
     p_currentrow=0;
+    p_currentcolumn=0;
     p_hk_kdesimplegridprivate=new hk_kdesimplegridprivate();
     p_combobox_created=false;
     setSelectionMode(Single);
@@ -274,8 +328,11 @@ hk_dsvisible(form)
     connect(horizontalHeader(), SIGNAL(indexChange(int,int,int)),this,SLOT(column_moved(int,int,int)));
     connect(verticalHeader(), SIGNAL(pressed(int)),this,SLOT(setFocus()));
     connect(verticalHeader(), SIGNAL(sizeChange(int,int,int)),this,SLOT(slot_verticalheader_resized(int,int,int)));
+    connect(this,SIGNAL(contentsMoving(int,int)),this,SLOT(slot_content_moving(int,int)));
     verticalHeader()->setTracking(false);
     setDragEnabled(true);
+    QFont f=QWidget::font();
+   cerr <<"fontsize:"<<f.pointSize()<<" rowheight:"<<p_grid->rowheight()<<endl;
 }
 
 
@@ -307,7 +364,11 @@ void hk_kdesimplegrid::widget_specific_enable_disable(void)
 #endif
     if (datasource()!=NULL)
     {
-    columns_created();
+    clear_vertical_header();
+    p_currentrow=0;
+    p_currentcolumn=0;
+    paint_vertical_header();
+    //columns_created();
     }
     else
     {
@@ -319,18 +380,25 @@ void hk_kdesimplegrid::widget_specific_enable_disable(void)
 void hk_kdesimplegrid::columns_created(void)
 {
 if (!datasource()) return;
+clear_vertical_header();
+
         setSorting(!datasource()->is_rawsql());
         if (datasource()->is_enabled())
         {
             if(datasource()->columns()!=NULL)
                 setNumCols(p_grid->columnscount());
-            setNumRows(datasource()->max_rows()+((datasource()->is_readonly()||p_grid->is_readonly())?0:1));
-            if (!datasource()->is_readonly()&&!p_grid->is_readonly())verticalHeader()->setLabel(datasource()->max_rows(),"*");
+		
+	   int newnumrows=datasource()->max_rows()+((datasource()->is_readonly()||p_grid->is_readonly())?0:1);	
+	   if (newnumrows>numRows())
+	    {
+   		QString r;
+    		r.setNum(numRows());
+    		verticalHeader()->setLabel(numRows()-1,no_image,r);
+	    
+	    }
+            setNumRows(newnumrows);
+            //paint_vertical_header();
             setEnabled(true);
-            QString r;
-            p_currentrow==datasource()->max_rows()?r="*":r.setNum(p_currentrow+1);
-            verticalHeader()->setLabel(p_currentrow,arrow_image,r);
-
             for(unsigned int count=0;count<p_grid->columnscount();count++)
             {
                 horizontalHeader()->setLabel(horizontalHeader()->mapToLogical(count),
@@ -354,6 +422,7 @@ if (!datasource()) return;
             setEnabled(false);
         }
 widget_specific_rowheight_changes();
+paint_vertical_header();
 }
 
 
@@ -398,6 +467,7 @@ void hk_kdesimplegrid::paintCell(QPainter* p,int row, int col, const QRect& cr,b
 void hk_kdesimplegrid::paint_textcell(QPainter* p,int row, int col, const QRect& ,bool selected , const QColorGroup&)
 {
     if (p_grid->gridcolumn(col)->column()==NULL) return;
+bool is_null=false;
     Qt::AlignmentFlags f;
     QPen pen=p->pen();
     bool currentcell=currentRow()==row && currentColumn()==col;
@@ -416,6 +486,7 @@ void hk_kdesimplegrid::paint_textcell(QPainter* p,int row, int col, const QRect&
         ((unsigned int)row==datasource()->max_rows()&&datasource()->mode()==hk_datasource::mode_insertrow)) )
     {
         txt=QString::fromLocal8Bit (p_grid->gridcolumn(col)->column()->changed_data_asstring(true).c_str());
+	is_null=p_grid->gridcolumn(col)->column()->changed_data_is_nullvalue();
     }
     else
     {
@@ -423,12 +494,18 @@ void hk_kdesimplegrid::paint_textcell(QPainter* p,int row, int col, const QRect&
         {
                                                   //&&col==currentColumn())
             if (p_grid->gridcolumn(col)->use_defaultvalue())
+	    {
                 txt=QString::fromLocal8Bit (p_grid->gridcolumn(col)->defaultvalue().c_str());
+		is_null=txt.isEmpty();
+	    }
         }
         else
+	{
             txt=QString::fromLocal8Bit (p_grid->gridcolumn(col)->column()->asstring_at(row,true).c_str());
+	    is_null=p_grid->gridcolumn(col)->column()->is_nullvalue_at(row);
+	}    
     }
-    if (is_numerictype(p_grid->gridcolumn(col)->column())&&p_grid->gridcolumn(col)->column()->columntype()!=hk_column::auto_inccolumn &&txt.length()>0
+    if ((!is_null&&is_numerictype(p_grid->gridcolumn(col)->column())&&p_grid->gridcolumn(col)->column()->columntype()!=hk_column::auto_inccolumn &&txt.length()>0)
         || (p_grid->gridcolumn(col)->column()->columntype()==hk_column::auto_inccolumn
         && (unsigned int)row!=datasource()->max_rows())
         )
@@ -502,7 +579,7 @@ void hk_kdesimplegrid::paint_boolcell(QPainter* painter,int row, int col, const
     int top=(rowHeight(row)-square)/2 ;
     qDrawShadeRect(painter,left,top,square,square,farbe,false,1);
 
-    bool draw=false;
+    int draw=0;
     bool currentcell=currentRow()==row && currentColumn()==col;
 
     if (p_grid->gridcolumn(col)!=NULL&& p_grid->gridcolumn(col)->column()!=NULL)
@@ -512,15 +589,29 @@ void hk_kdesimplegrid::paint_boolcell(QPainter* painter,int row, int col, const
             draw=p_grid->gridcolumn(col)->column()->changed_data_asbool();
         }
         else
-            draw=p_grid->gridcolumn(col)->column()->asbool_at(row);
+	{
+            if (p_grid->gridcolumn(col)->column()->is_nullvalue_at(row)) draw=2;
+	    else
+	    if (p_grid->gridcolumn(col)->column()->asbool_at(row)) draw=1;
+	}    
     }
-    if (draw)
+    switch (draw)
     {
+    case 1: { // TRUE
 
         const QBrush brush =QBrush(selected&&!(currentcell && hasFocus())?colorGroup().highlightedText():colorGroup().text());
         qDrawPlainRect(painter,left+3,top+3,square/2,square/2,colorGroup().dark(),1,&brush);
-    }
-
+            break;
+	    }
+   case 2:	 {// NULL Value
+         QColor nullcolor=colorGroup().text();
+        QBrush brush =QBrush(nullcolor);
+	brush.setStyle(QBrush::Dense5Pattern);
+        qDrawPlainRect(painter,left+2,top+2,square-4,square-4,nullcolor,0,&brush);
+            break;
+	    }
+   default:;	        
+   }
 }
 
 
@@ -551,6 +642,8 @@ QWidget* hk_kdesimplegrid::createEditor(int row, int col,bool ) const
             e=i;
             i->set_datasource(p_grid->datasource());
             i->set_columnname(p_grid->gridcolumn(col)->columnname());
+	    i->set_gridcolumn(p_grid->gridcolumn(col));
+
             break;
         }
         case hk_dsgridcolumn::columncombo:
@@ -574,6 +667,8 @@ QWidget* hk_kdesimplegrid::createEditor(int row, int col,bool ) const
             l->set_columnname(p_grid->gridcolumn(col)->columnname());
             l->set_readonly((isReadOnly() || isRowReadOnly( row ) || isColumnReadOnly( col )||p_grid->gridcolumn(col)->is_readonly()));
             l->set_font(p_grid->hk_dsgrid::font());
+            l->set_gridcolumn(p_grid->gridcolumn(col));
+
 
             if (p_grid->presentation())
                 l->set_listdatasource(p_grid->presentation()->get_datasource(c->listdatasourcename()));
@@ -582,7 +677,7 @@ QWidget* hk_kdesimplegrid::createEditor(int row, int col,bool ) const
                 if (p_grid->datasource())
                 {
                     hk_database* db=p_grid->datasource()->database();
-                    hk_datasource* ds=db->load_datasource(c->listdatasourcename(),!c->listdatasource_is_table());
+                    hk_datasource* ds=db->load_datasource(c->listdatasourcename(),c->listdatasourcetype());
                     l->set_listdatasource(ds);
                     if (ds)ds->enable();
                 }
@@ -608,6 +703,8 @@ QWidget* hk_kdesimplegrid::createEditor(int row, int col,bool ) const
     l->set_font(p_grid->hk_dsgrid::font());
     l->set_readonly((isReadOnly() || isRowReadOnly( row ) || isColumnReadOnly( col )||p_grid->gridcolumn(col)->is_readonly()));
     l->selectAll();
+    l->set_gridcolumn(p_grid->gridcolumn(col));
+
     e=l;
 
 	 }
@@ -616,6 +713,7 @@ QWidget* hk_kdesimplegrid::createEditor(int row, int col,bool ) const
 	if (!e)e=create_lineeditor(row,col);
 	}
     }
+    
     return e;
 
 }
@@ -629,11 +727,13 @@ QWidget* hk_kdesimplegrid::create_lineeditor(int row, int col) const
     l->set_alignment(p_grid->gridcolumn(col)->alignment());
     l->set_numberformat(p_grid->gridcolumn(col)->use_numberseparator(),p_grid->gridcolumn(col)->commadigits());
     l->set_datasource(p_grid->datasource());
+    l->set_presentationdatasource(p_grid->presentationdatasource());
     l->set_defaultvalue(p_grid->gridcolumn(col)->defaultvalue());
     l->set_columnname(p_grid->gridcolumn(col)->columnname());
     l->set_font(p_grid->hk_dsgrid::font());
     l->set_readonly((isReadOnly() || isRowReadOnly( row ) || isColumnReadOnly( col )||p_grid->gridcolumn(col)->is_readonly()));
     l->selectAll();
+    l->set_gridcolumn(p_grid->gridcolumn(col));
     return l;
 
 }
@@ -682,35 +782,43 @@ void    hk_kdesimplegrid::previouscell()
 }
 
 
-void hk_kdesimplegrid::row_changed(int row,int /*col*/)
+void hk_kdesimplegrid::row_changed(int row,int col)
 {
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimplegrid::row_changed");
 #endif
+//cerr <<"row_changed"<<endl;
    if (!datasource()) return;
-    QString r;
-    QString a="*";
-    r.setNum(p_currentrow+1);
-    if (!datasource()->is_readonly()&& !p_grid->is_readonly())
-        verticalHeader()->setLabel(datasource()->max_rows(),a);
-    verticalHeader()->setLabel(p_currentrow,no_image,
-        ((!datasource()->is_readonly()&&!p_grid->is_readonly())&&p_currentrow==datasource()->max_rows())?a:r);
-    r.setNum(row+1);
-    verticalHeader()->setLabel(row,arrow_image,
-        (!(datasource()->is_readonly()&&!p_grid->is_readonly())&&(unsigned int)row==datasource()->max_rows())?a:r);
-
-    p_currentrow=row;
+   if (!datasource()->is_enabled()) return;
+   if (p_grid->gridcolumn(p_currentcolumn))
+    p_grid->gridcolumn(p_currentcolumn)->action_on_loosefocus();
+    
+    clear_vertical_header();
+    bool result=true;
     if (datasource()!=NULL)
     {
         if (datasource()->is_enabled())
         {
             if ((unsigned)row<datasource()->max_rows())
-                datasource()->goto_row(row);
+                result=datasource()->goto_row(row);
             else  if (datasource()->mode()!=hk_datasource::mode_insertrow)
                 datasource()->setmode_insertrow();
         }
     }
+    if (!result)
+    {
+     setCurrentCell(p_currentrow,currentColumn());
+     paint_vertical_header();
+     return;
+    }
+    endEdit(p_currentrow,p_currentcolumn,true,false);   
+    
+
+    p_currentrow=row;
+    p_currentcolumn=col;
+    paint_vertical_header();
     set_statustext();
+    p_grid->gridcolumn(currentColumn())->action_on_getfocus();;
 //clearSelection(true);
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimplegrid::row_changed ENDE");
@@ -768,12 +876,14 @@ void hk_kdesimplegrid::widget_specific_row_added()
 #ifdef HK_DEBUG
     hkdebug ("hk_kdesimplegrid::widget_specific_row_added");
 #endif
+    clear_vertical_header();
     if (datasource()!=NULL)
     {
         setNumRows(datasource()->max_rows()+((datasource()->is_readonly()||p_grid->is_readonly())?0:1));
     }
     else setNumRows(1);
 	widget_specific_rowheight_changes();
+ paint_vertical_header();
 }
 
 
@@ -782,12 +892,13 @@ void hk_kdesimplegrid::widget_specific_row_deleted()
 #ifdef HK_DEBUG
     hkdebug ("hk_kdesimplegrid::widget_specific_row_deleted");
 #endif
+    clear_vertical_header();
     if (datasource()!=NULL)
     {
         setNumRows(datasource()->max_rows()+((datasource()->is_readonly()||p_grid->is_readonly())?0:1));
     }
     else setNumRows(1);
-
+paint_vertical_header();
 }
 
 
@@ -843,6 +954,7 @@ bool hk_kdesimplegrid::datasource_disable(void)
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimplegrid::datasource_disable");
 #endif
+    cerr <<"datasource_disable()"<<endl;
     ensureCellVisible(0,0);
     widget_specific_enable_disable();
     setNumCols(1);
@@ -905,11 +1017,25 @@ void hk_kdesimplegrid::keyPressEvent(QKeyEvent* e)
 
 			break;
             }
+	    case Key_0:
+	    {
+		    hk_column* c=p_grid->gridcolumn(currentColumn())->column();
+    		    if (c) c->set_asnullvalue();
+	    }
 	    default:;
 
         }
 
-    }                                             // ende->state()&ControlButton
+    }
+   // ende->state()&ControlButton
+   else
+   {
+    if (e->key()==Key_Escape && datasource())
+      {
+        datasource()->reset_changed_data();
+	widget_specific_row_change();
+      }
+   }
 
 }
 
@@ -998,14 +1124,18 @@ void hk_kdesimplegrid::show_contextmenu(int , int , const QPoint& pos)
     if (datasource()==NULL)return;
     QPopupMenu* popup=new QPopupMenu(this);
     int id[8];
-    id[ 4 ] = popup->insertItem( i18n( "Define columns" ) );
+    
+    if (runtime_only()) id[4]=-99;
+    else
+      id[ 4 ] = popup->insertItem( i18n( "Define columns" ) );
+    
     id[ 5 ] = popup->insertItem( i18n( "Find" ) );
     popup->insertSeparator();
     id[ 3 ] = popup->insertItem( i18n( "Delete row" ) );
     hk_column* c=p_grid->gridcolumn(currentColumn())->column();
     if (c  && !c->is_readonly()&&!c->is_notnull())
     {
-        id[ 6 ] = popup->insertItem( i18n( "set 'NULL' value" ) );
+        id[ 6 ] = popup->insertItem( i18n( "set 'NULL' value" ),CTRL+Key_0 );
     }
     int r = popup->exec( pos );
 
@@ -1027,7 +1157,10 @@ void hk_kdesimplegrid::show_gridcolumndialog(void)
 {
     hk_kdegridcolumndialog* dialog=new hk_kdegridcolumndialog(p_grid,0,true);
 
-    dialog->exec();
+    if (dialog->exec()==QDialog::Accepted)
+    {
+     p_grid->has_changed(true,forcesetting);
+    }
     widget_specific_enable_disable();
 
     delete dialog;
@@ -1083,6 +1216,7 @@ bool hk_kdesimplegrid::delete_selected_rows(void)
     hkdebug("hk_kdesimplegrid::delete_selected_rows");
 #endif
     if (!datasource()) return false;
+    if (datasource()->is_readonly()) return false;
     if (isRowSelected(selection(0).topRow(),true))
     {
         bool res=datasource()->delete_rows(selection(0).topRow(),selection(0).bottomRow());
@@ -1446,27 +1580,37 @@ void hk_kdesimplegrid::paste_tsv(const QString& txt)
 
 
 void hk_kdesimplegrid::slot_mouse_clicked(void)
-{
+{  
    if (p_presentation && p_presentation->mode()==hk_presentation::viewmode
        )
+   {    
           p_grid->action_on_click();
+	  p_grid->gridcolumn(currentColumn())->action_on_click();
+   }	  
 }
 
 void hk_kdesimplegrid::slot_mouse_doubleclicked(void)
 {
    if (p_presentation && p_presentation->mode()==hk_presentation::viewmode
        )
-          p_grid->action_on_click();
+   {
+          p_grid->action_on_doubleclick();
+  	  p_grid->gridcolumn(currentColumn())->action_on_doubleclick();
+
+   }
 }
 
 
-void hk_kdesimplegrid::focusOutEvent ( QFocusEvent * )
+void hk_kdesimplegrid::focusOutEvent ( QFocusEvent * e )
 {
+action_on_loosefocus();
  if (p_grid&&p_grid->part())
-   emit signal_outfocus(p_grid->part());
+   emit signal_outfocus(p_grid->part(),e);
 }
+
 void hk_kdesimplegrid::focusInEvent ( QFocusEvent * )
 {
+action_on_getfocus();
 
 if (p_grid&&p_grid->part())
    emit signal_infocus(p_grid->part());
@@ -1546,14 +1690,66 @@ p_hk_kdesimplegridprivate->p_resizeverticalheader=false;
 
 void hk_kdesimplegrid::widget_specific_rowheight_changes(void)
 {//cerr<<"hk_kdesimplegrid::widget_specific_rowheight_changes "<<p_hk_kdesimplegridprivate->newsize<<endl;
+//disabled because QTable behaves terrible for large tables
+//return;
+
 p_hk_kdesimplegridprivate->autoresizeverticalheader=true;
- for (int i=0;i<=numRows();++i)
-   {
-     setRowHeight(i,p_grid->rowheight());
-   }
+int begin=rowAt(contentsY());
+slot_content_moving(0,begin);
 p_hk_kdesimplegridprivate->autoresizeverticalheader=false;
 }
 
 
 
+void hk_kdesimplegrid::slot_content_moving(int /*x*/,int y)
+{
+int begin=rowAt(y);
+int end=rowAt(y+QTable::height());
+if (end<begin||end>numRows()) end=numRows();
+if (end+500>numRows()) end=numRows();
+if (end<numRows()+100) end+=100;
+
+cerr <<"slot_content_moving b="<<begin<<" e="<<end<<endl;
+ for (int i=begin;i<=end;++i)
+   {
+     if (rowHeight(i)!=p_grid->rowheight())
+       setRowHeight(i,p_grid->rowheight());
+   }
+
+
+}
+
+
+hk_kdegrid* hk_kdesimplegrid::kdegrid() const
+{
+ return p_grid;
+}
+
+
+void  hk_kdesimplegrid::widget_specific_tooltip_changed(void)
+{
+ QToolTip::add(this,QString::fromLocal8Bit (tooltip().c_str()));
+}
+
+
+void hk_kdesimplegrid::clear_vertical_header(void)
+{
+   if (datasource() &&!datasource()->is_readonly()&&!p_grid->is_readonly())
+   
+     verticalHeader()->setLabel(datasource()->max_rows(),"*");
+   QString r;
+    p_currentrow==datasource()->max_rows()?r="*":r.setNum(p_currentrow+1);
+    verticalHeader()->setLabel(p_currentrow,no_image,r);
+
+}
+
+void hk_kdesimplegrid::paint_vertical_header(void)
+{
+   if (datasource() &&!datasource()->is_readonly()&&!p_grid->is_readonly())
+        verticalHeader()->setLabel(datasource()->max_rows(),"*");
+   QString r;
+    p_currentrow==datasource()->max_rows()?r="*":r.setNum(p_currentrow+1);
+    verticalHeader()->setLabel(p_currentrow,arrow_image,r);
+}
+
 
