@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,13 +8,15 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.58 $
+//$Revision: 1.70 $
 
 #include "hk_kdesimplereport.h"
 #include "hk_kdereportsection.h"
 #include "hk_kdereportproperty.h"
 #include "hk_kdereport.h"
+#include "../hk_kdereportpart/hk_kdereportpart.h"
 #include "hk_kdereportdata.h"
+#include "hk_kdeformfocus.h"
 #include <hk_reportsectionpair.h>
 #include <qlayout.h>
 #include <hk_kdeformfocus.h>
@@ -51,11 +53,12 @@ bool while_modechange;
         hk_kdereportproperty* p_property;
         QVBoxLayout* p_reportlayout;
         hk_kdereport* p_kdereport;
+	hk_kdereportpartwidget* p_reportpartwidget;
         hk_kdeformfocus* p_focus;
         hk_kdereportsection* p_focussection;
         list<hk_kdeformfocus*> p_multiplefocus;
         QPoint p_originalposition;
-        bool p_controlbutton;
+        //bool p_controlbutton;
         bool p_autoclose;
         bool p_sourcevanishflag;
         bool p_createfield;
@@ -125,6 +128,7 @@ hk_kdesimplereport::hk_kdesimplereport (QWidget* parent,  const char* name, WFla
     p_private->p_autoclose=true;
     p_private->p_createfield=false;
     p_private->p_kdereport=NULL;
+    p_private->p_reportpartwidget=NULL;
     init_report();
     set_sections();
     setFocusPolicy(StrongFocus);
@@ -158,6 +162,20 @@ hk_kdesimplereport::~hk_kdesimplereport(void)
 
 }
 
+void hk_kdesimplereport::set_reportpartwidget(hk_kdereportpartwidget* w)
+{
+
+  p_private->p_reportpartwidget=w;
+    if (p_private->p_property) 
+      {
+       delete p_private->p_property;
+       p_private->p_property=NULL;
+      } 
+  
+  show_property();
+}
+
+
 
 void hk_kdesimplereport::set_kdereport(hk_kdereport* k)
 {
@@ -334,15 +352,19 @@ bool hk_kdesimplereport::reporteventFilter(QObject* object,QEvent* event)
                 hk_kdereportsection* s=NULL;
                 hk_kdereportdata* d=dynamic_cast<hk_kdereportdata*>(object);
                 if (d!=NULL) s=d->section();
-                set_focus(widget,s);
+                set_focus(widget,s,((QMouseEvent*)event)->state() &ControlButton);
 
                 if (widget!=NULL&&p_private->p_focus!=NULL)p_private->p_focus->show();
 // the following allows the label widget of a hk_kdereportsection to
 // activate the section on a mousepressevent
                 if (!s && widget) return QWidget::eventFilter(object,event);
                 p_private->p_originalposition=mapFromGlobal(((QMouseEvent*)event)->globalPos());
-		if (snap2gridx()>0&&p_private->p_originalposition.x()%snap2gridx()>0&&!ctrl_key_pressed())p_private->p_originalposition.setX(p_private->p_originalposition.x()-p_private->p_originalposition.x()%snap2gridx());
-		if (snap2gridy()>0&&p_private->p_originalposition.y()%snap2gridy()>0&&!ctrl_key_pressed())p_private->p_originalposition.setY(p_private->p_originalposition.y()-p_private->p_originalposition.y()%snap2gridy());
+		
+		if (snap2gridx()>0&&!(((QMouseEvent*)event)->state() & ControlButton))
+		p_private->p_originalposition.setX((p_private->p_originalposition.x()/snap2gridx())*snap2gridx());
+		
+		if (snap2gridy()>0&&!(((QMouseEvent*)event)->state() & ControlButton))
+		p_private->p_originalposition.setY((p_private->p_originalposition.y()/snap2gridy())*snap2gridy());
                 return true;
             }
             else
@@ -363,8 +385,13 @@ bool hk_kdesimplereport::reporteventFilter(QObject* object,QEvent* event)
             {
 //move object
                 QPoint newpos=mapFromGlobal(((QMouseEvent*)event)->globalPos());
-		if (snap2gridx()>0&&newpos.x()%snap2gridx()>0&&!ctrl_key_pressed())newpos.setX(newpos.x()-newpos.x()%snap2gridx());
-		if (snap2gridy()>0&&newpos.y()%snap2gridy()>0&&!ctrl_key_pressed())newpos.setY(newpos.y()-newpos.y()%snap2gridy());
+		
+		if (snap2gridx()>0&&!(((QMouseEvent*)event)->state() & ControlButton))
+		newpos.setX((newpos.x()/snap2gridx())*snap2gridx());
+		
+		if (snap2gridy()>0&&!(((QMouseEvent*)event)->state() & ControlButton))
+		newpos.setY((newpos.y()/snap2gridy())*snap2gridy());
+		
                 QPoint difference=p_private->p_originalposition-newpos;
 		QPoint buf=p_private->p_originalposition;
                 p_private->p_originalposition=newpos;
@@ -451,7 +478,7 @@ bool hk_kdesimplereport::reporteventFilter(QObject* object,QEvent* event)
 }
 
 
-void hk_kdesimplereport::set_focus(QWidget*f,hk_kdereportsection* s)
+void hk_kdesimplereport::set_focus(QWidget*f,hk_kdereportsection* s, bool ctrl)
 {
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimplereport::set_focus");
@@ -459,6 +486,7 @@ void hk_kdesimplereport::set_focus(QWidget*f,hk_kdereportsection* s)
     if (f==NULL || f==this ||s==NULL)
     {
         clearfocus();
+	emit signal_focuswidget_changed();
         return;
 
     }
@@ -474,7 +502,7 @@ void hk_kdesimplereport::set_focus(QWidget*f,hk_kdereportsection* s)
     }
     set_actualsection(s);
     if (p_private->p_focus==NULL) p_private->p_focus=s->new_focus();
-    if (p_private->p_controlbutton==true)
+    if (ctrl==true)
     {
 
         if (p_private->p_focus->widget()==f ) return;
@@ -488,7 +516,7 @@ void hk_kdesimplereport::set_focus(QWidget*f,hk_kdereportsection* s)
 
     }
 
-    if (p_private->p_controlbutton==true&& p_private->p_focus->widget()!=NULL &&p_private->p_focus->widget()!=this)
+    if (ctrl==true&& p_private->p_focus->widget()!=NULL &&p_private->p_focus->widget()!=this)
     {
         p_private->p_multiplefocus.insert(p_private->p_multiplefocus.end(),p_private->p_focus);
         hk_kdeformfocus* pf=p_private->p_focus;
@@ -501,6 +529,8 @@ void hk_kdesimplereport::set_focus(QWidget*f,hk_kdereportsection* s)
     p_private->p_focus->set_widget(f);
     set_propertyobject(f);
     enable_actions();
+emit signal_focuswidget_changed();
+
 }
 
 
@@ -538,7 +568,7 @@ void hk_kdesimplereport::clearfocus(void)
         p_private->p_focus=NULL;
 
     }
-    p_private->p_controlbutton=false;
+    //p_private->p_controlbutton=false;
     clearmultiplefocus();
     enable_actions();
 }
@@ -586,10 +616,18 @@ void hk_kdesimplereport::before_source_vanishes(void)
  {
    reparent(0L,WDestructiveClose,QPoint(0,0));
     if (p_private->p_kdereport)
-      {hk_kdereport* k=p_private->p_kdereport;
+      {
+       hk_kdereport* k=p_private->p_kdereport;
        set_kdereport(NULL);
        k->close();
       } 
+    else
+        if (p_private->p_reportpartwidget)
+      {
+       hk_kdereportpartwidget* k=p_private->p_reportpartwidget;
+       set_reportpartwidget(NULL);
+       k->close();
+      } 
     else p->close();
    close();
 
@@ -622,6 +660,12 @@ void hk_kdesimplereport::database_has_vanished(void)
        set_kdereport(NULL);
        k->close();
       } 
+    else
+     if (p_private->p_reportpartwidget)
+      {hk_kdereportpartwidget* k=p_private->p_reportpartwidget;
+       set_reportpartwidget(NULL);
+       k->close();
+      } 
     else p->close();
    close();
  }
@@ -667,6 +711,18 @@ bool hk_kdesimplereport::set_mode(enum_mode m)
 		else
  		p_private->p_kdereport->set_mode(m);
     }
+    else
+    if (p_private->p_reportpartwidget)
+    {
+   if (m==viewmode&&!r)
+   		{
+		hk_report::set_mode(designmode);
+ 		p_private->p_reportpartwidget->set_mode(designmode);
+		}
+		else
+ 		p_private->p_reportpartwidget->set_mode(m);
+    }
+
     p_private->while_modechange=false;
      return r;
 
@@ -726,10 +782,10 @@ void hk_kdesimplereport::widget_specific_modechanges(enum_mode s)
     enable_actions();
 }
 
-bool hk_kdesimplereport::ctrl_key_pressed(void) const
+/*bool hk_kdesimplereport::ctrl_key_pressed(void) const
 {
 return p_private->p_controlbutton;
-}
+}*/
 
 void hk_kdesimplereport::keyPressEvent ( QKeyEvent * event )
 {
@@ -739,17 +795,18 @@ void hk_kdesimplereport::keyPressEvent ( QKeyEvent * event )
    const unsigned int pix_je_cm=32 ;
      switch(event->key())
     {
-        case Key_Control:  p_private->p_controlbutton=true;
-        break;
+//        case Key_Control:  p_private->p_controlbutton=true;
+//        break;
         case Key_Delete:   delete_widgets();
         break;
+   
    case Key_Left:
             if (p_private->p_focus &&p_private->p_focus->widget())
             {
               hk_visible* widget=dynamic_cast<hk_visible*>(p_private->p_focus->widget());
               if (widget)
               {
-               unsigned int diff=(p_private->p_controlbutton?1:snap2gridx());
+               unsigned int diff=(event->state()&ControlButton?1:snap2gridx());
                diff=(unsigned int)(sizetype()==hk_presentation::relative?
 	       				(p_private->p_focussection?((double)diff*10000)/ p_private->p_focussection->max_displaywidth()+0.5:diff):
 		     			((double)diff*100)/pix_je_cm+0.5);
@@ -758,13 +815,15 @@ void hk_kdesimplereport::keyPressEvent ( QKeyEvent * event )
 
             }
 break;
+
+
 case Key_Right:
 if (p_private->p_focus &&p_private->p_focus->widget())
 {
 hk_visible* widget=dynamic_cast<hk_visible*>(p_private->p_focus->widget());
 if (widget)
 {
-unsigned int diff=(p_private->p_controlbutton?1:snap2gridx());
+unsigned int diff=(event->state()&ControlButton?1:snap2gridx());
                diff=(unsigned int)(sizetype()==hk_presentation::relative?
 	       				(p_private->p_focussection?((double)diff*10000)/ p_private->p_focussection->max_displaywidth()+0.5:diff):
 		     			((double)diff*100)/pix_je_cm+0.5);
@@ -773,13 +832,15 @@ if (widget->x()+widget->width()+diff<=10000) widget->set_x(widget->x()+diff);
 
 }
 break;
+
+
 case Key_Up:
 if (p_private->p_focus &&p_private->p_focus->widget())
 {
 hk_visible* widget=dynamic_cast<hk_visible*>(p_private->p_focus->widget());
 if (widget)
 {
-unsigned int diff=(p_private->p_controlbutton?1:snap2gridy());
+unsigned int diff=(event->state()&ControlButton?1:snap2gridy());
                diff=(unsigned int)(sizetype()==hk_presentation::relative?
 	       				(p_private->p_focussection?((double)diff*10000)/ p_private->p_focussection->max_displayheight()+0.5:diff):
 		     			((double)diff*100)/pix_je_cm+0.5);
@@ -788,13 +849,15 @@ if (widget->y()>=diff) widget->set_y(widget->y()-diff);
 
 }
 break;
+
+
 case Key_Down:
 if (p_private->p_focus &&p_private->p_focus->widget())
 {
 hk_visible* widget=dynamic_cast<hk_visible*>(p_private->p_focus->widget());
 if (widget &&p_private->p_focussection)
 {
-unsigned int diff=(p_private->p_controlbutton?1:snap2gridy());
+unsigned int diff=(event->state()&ControlButton?1:snap2gridy());
                diff=(unsigned int)(sizetype()==hk_presentation::relative?
 	       				(p_private->p_focussection?((double)diff*10000)/ p_private->p_focussection->max_displayheight()+0.5:diff):
 		     			((double)diff*100)/pix_je_cm+0.5);
@@ -828,7 +891,7 @@ void hk_kdesimplereport::keyReleaseEvent ( QKeyEvent * event)
 #ifdef HK_DEBUG
     hkdebug("hk_kdesimplereport::keyReleaseEvent");
 #endif
-    if (event->key()==Key_Control)  p_private->p_controlbutton=false;
+   // if (event->key()==Key_Control)  p_private->p_controlbutton=false;
     QWidget::keyReleaseEvent(event);
 
 }
@@ -1026,9 +1089,10 @@ bool hk_kdesimplereport::widget_specific_after_new_page(void)
 {
     if (p_private->p_progressdialog!=NULL)
     {
-        p_private->p_progressdialog->setProgress(datasource()->row_position());
+        if (datasource())
+	  p_private->p_progressdialog->setProgress(datasource()->row_position());
     }
-    qApp->processEvents();
+    if (p_private->p_reportpartwidget)qApp->processEvents();
 
     return !p_private->p_cancelprinting;
 }
@@ -1318,6 +1382,7 @@ void hk_kdesimplereport::show_property(void)
     if (p_private->p_property==NULL)
     {   
          p_private->p_property=new hk_kdereportproperty(this,0,false);
+	 p_private->p_property->set_object(this);
          if (p_private->p_kdereport)
         {
 	 QPixmap p;
@@ -1325,6 +1390,15 @@ void hk_kdesimplereport::show_property(void)
 	 p_private->p_propertydock->setWidget(p_private->p_property);
 	 p_private->p_propertydock->manualDock(p_private->p_kdereport->getMainDockWidget(),KDockWidget::DockRight);
 	}
+	else
+	if (p_private->p_reportpartwidget)
+        {
+	 QPixmap p;
+	 p_private->p_propertydock=p_private->p_reportpartwidget->createDockWidget("Property editor",p,0L,"");
+	 p_private->p_propertydock->setWidget(p_private->p_property);
+	 p_private->p_propertydock->manualDock(p_private->p_reportpartwidget->getMainDockWidget(),KDockWidget::DockRight);
+	}
+
        newproperty=true;
 
     }
@@ -1413,7 +1487,7 @@ void hk_kdesimplereport::paste_widgets(void)
     if (!QApplication::clipboard()->data()) return;
     if (!QApplication::clipboard()->data()->encodedData("application/x-hk_kdesimplereportcopy"))return ;
     clearfocus();
-    p_private->p_controlbutton=true;
+    //p_private->p_controlbutton=true;
     hk_string definition=u2l(QApplication::clipboard()->data()->encodedData("application/x-hk_kdesimplereportcopy").data());
     int i=1;
     hk_string value;
@@ -1436,7 +1510,7 @@ void hk_kdesimplereport::paste_widgets(void)
             QWidget* w=dynamic_cast<QWidget*>(vis);
             if (w)
             {
-                set_focus(w,rs);
+                set_focus(w,rs,true);
                 w->repaint(true);
             }
         }
@@ -1649,20 +1723,22 @@ if (dv)
 
 if (!runtime_only())
    {
-      kdereport()->set_mode(hk_presentation::designmode);
-   	 if (p_private->p_property)
+      if (kdereport())kdereport()->set_mode(hk_presentation::designmode);
+      if (p_private->p_reportpartwidget) 
+         p_private->p_reportpartwidget->set_mode(hk_presentation::designmode);
+      if (p_private->p_property)
 	    {
 
                 hk_kdereportsection* s=NULL;
                 hk_kdereportdata* d=dynamic_cast<hk_kdereportdata*>(v);
                 if (d!=NULL) s=d->section();
-	       set_focus(d,s);
+	       set_focus(d,s,false);
 	     switch (a)
 	     {
-		case hk_interpreter::a_onopen :
+		case hk_interpreter::a_on_open :
 				p_private->p_property->openactionbutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
 				break;
-		case hk_interpreter::a_onclose :
+		case hk_interpreter::a_on_close :
 				p_private->p_property->closeactionbutton_clicked(presentation()->interpreter()->error_rownumber()-1,error);
 				break;
 		case hk_interpreter::a_after_row_change :
@@ -1694,3 +1770,26 @@ void    hk_kdesimplereport::widget_specific_fieldresize(hk_visible* v)
  if (!v||!p_private->p_property) return;
  if (v==p_private->p_property->object()) p_private->p_property->set_object(v);
 }
+
+
+
+bool hk_kdesimplereport::focus_multipleselected(void) const
+{
+return p_private->p_multiplefocus.size()>0;
+}
+
+
+void hk_kdesimplereport::set_has_changed(enum_has_changed forcesetting)
+{
+hk_report::set_has_changed(forcesetting);
+emit signal_has_changed();
+}
+
+
+
+void hk_kdesimplereport::reset_has_changed(void)
+{
+  hk_report::reset_has_changed();
+emit signal_has_changed();
+}
+
