@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.34 $
+//$Revision: 1.39 $
 
 #include "hk_kdetabledesign.h"
 #include "hk_kdetabledesign.moc"
@@ -62,6 +62,7 @@ hk_kdetabledesign::hk_kdetabledesign( QWidget* parent,  const char* name, WFlags
     indexwindow->setAllColumnsShowFocus(true);
 
     fieldlist = new QListView( this, "fieldlist" );
+    fieldlist->setSorting(-1);
     fieldlist->addColumn( i18n( "Name" ) );
     fieldlist->addColumn( i18n( "Type" ) );
     fieldlist->addColumn( i18n( "Size" ) );
@@ -279,7 +280,17 @@ void hk_kdetabledesign::create_columntypelist(void)
         )
         )
     {
-        setEnabled(false);
+        fieldlist->setEnabled(false);
+	buttonlayout->setEnabled(false);
+	typefield->setEnabled(false);
+	sizefield->setEnabled(false);
+	primaryfield->setEnabled(false);
+	notnullfield->setEnabled(false);
+	namefield->setEnabled(false);
+	deletebutton->setEnabled(false);
+	
+	
+	
     }
     if (datasource()!=NULL)
         if (datasource()->database()->connection()->server_supports(hk_connection::SUPPORTS_MEMOCOLUMN))
@@ -450,6 +461,7 @@ bool hk_kdetabledesign::alter_table(bool ask)
     {
         hk_string res=show_stringvaluedialog(hk_translate("Please enter the tablename: "));
         datasource()->set_name(res);
+	emit signal_tablename_changed();
         createtable=true;
     }
     if (datasource()->name().size()==0) return false;
@@ -750,7 +762,7 @@ void hk_kdetabledesign::selection_changed()
     if (actualfieldit!=fields.end())
         notnullfield->setChecked((*actualfieldit).newnotnull);
 
-    deletebutton->setEnabled(true);
+    deletebutton->setEnabled(namefield->isEnabled());
     namefield->blockSignals(false);
     sizefield->blockSignals(false);
     primaryfield->blockSignals(false);
@@ -829,11 +841,30 @@ void hk_kdetabledesign::load_fieldinfo()
     set_indexlist();
     if (fields.size()>0)
     {
-        setEnabled(datasource()->database()->connection()->server_supports(hk_connection::SUPPORTS_ALTER_TABLE));
+        bool res=datasource()->database()->connection()->server_supports(hk_connection::SUPPORTS_ALTER_TABLE);
+        fieldlist->setEnabled(res);
+	buttonlayout->setEnabled(res);
+	typefield->setEnabled(res);
+	sizefield->setEnabled(res);
+	primaryfield->setEnabled(res);
+	notnullfield->setEnabled(res);
+	namefield->setEnabled(res);
+	newbutton->setEnabled(res);
+	deletebutton->setEnabled(res);
+
+
     }
     else
     {
-       setEnabled(datasource()->database()->connection()->server_supports(hk_connection::SUPPORTS_NEW_TABLE));
+        bool res=datasource()->database()->connection()->server_supports(hk_connection::SUPPORTS_NEW_TABLE);
+        fieldlist->setEnabled(res);
+	buttonlayout->setEnabled(res);
+	typefield->setEnabled(res);
+	sizefield->setEnabled(res);
+	primaryfield->setEnabled(res);
+	notnullfield->setEnabled(res);
+	namefield->setEnabled(res);
+	newbutton->setEnabled(res);
     }
 
 
@@ -925,6 +956,7 @@ void hk_kdetabledesign::structure_changes(bool c)
 {
     p_has_changed=c;
     alterbutton->setEnabled(c);
+    if (c) emit signal_has_changed();
     indexnewbutton->setEnabled(!c);
     indexalterbutton->setEnabled(!c);
 
@@ -938,14 +970,21 @@ void hk_kdetabledesign::structure_changes(bool c)
     }
     else
     {
-        res=true;
+       if (fields.size()==0||c)
+        res=(datasource()&&
+        (datasource()->database()->connection()->
+	server_supports(hk_connection::SUPPORTS_NEW_TABLE)));
+      else  	
+        res=(datasource()&&
+        (datasource()->database()->connection()->
+	server_supports(hk_connection::SUPPORTS_ALTER_TABLE)));
     }
     namefield->setEnabled(res);
     typefield->setEnabled(res);
     primaryfield->setEnabled(res);
     notnullfield->setEnabled(res);
     sizefield->setEnabled(res);
-    deletebutton->setEnabled(res);
+    deletebutton->setEnabled(res); 
 
 }
 
@@ -1017,3 +1056,10 @@ void hk_kdetabledesign::set_fields_enabled(void)
 
 }
 
+
+bool hk_kdetabledesign::has_changed(void) const
+{
+   return p_has_changed;
+}
+
+
