@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of the hk_kdeclasses library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -21,7 +21,9 @@ class hk_kdexmlexportdialog : public hk_kdexmlexportdialogbase ,public hk_report
         public:
         hk_kdexmlexportdialog(const QString& table, QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
         ~hk_kdexmlexportdialog();
-        void set_usetablelist(bool tables);
+        void set_usetablelist(void);
+        void set_usequerylist(void);
+        void set_useviewlist(void);
 
     public slots:
         void ok_clicked();
@@ -34,7 +36,7 @@ class hk_kdexmlexportdialog : public hk_kdexmlexportdialogbase ,public hk_report
         void set_datasourcelist(void);
         QString      p_file;
         QString p_tablename;
-        bool p_usetablelist;
+	datasourcetype p_datasourcetype;
 
 };
 #endif                                            // HK_KDEXMLEXPORTDIALOG_H
