@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2004 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
 // This file is part of knoda -Knorr's Datenbank.
 // This file may be distributed and/or modified under the terms of the
 // GNU General Public License version 2 as published by the Free Software
@@ -8,10 +8,11 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.40 $
+//$Revision: 1.46 $
 
 #include "knodawinbase.h"
 #include "knodanewdatabasedialog.h"
+#include "knodamaindockwindow.h"
 #include <kstandarddirs.h>
 #include <kglobalsettings.h>
 
@@ -34,15 +35,18 @@
 #include <qapplication.h>
 #include <qprogressdialog.h>
 #include <qclipboard.h>
+#include <qstringlist.h>
 #include <kmenubar.h>
 #include <klocale.h>
 #include <kiconloader.h>
+#include <kfiledialog.h>
 
 
 #include <vector>
 
 #include <hk_class.h>
 #include <hk_string.h>
+#include <hk_url.h>
 #include "hk_kdetable.h"
 #include "hk_kdequery.h"
 #include "hk_kdegrid.h"
@@ -223,6 +227,8 @@ hk_string hk_listbox::create_draginfo(void)
     if (p_window->tablebutton->isChecked()) mystring+="TABLE";
         else
         if (p_window->querybutton->isChecked()) mystring+="QUERY";
+    else
+        if (p_window->viewbutton->isChecked()) mystring+="VIEW";
             else
             if (p_window->formbutton->isChecked()) mystring+="FORM";
                 else mystring+="REPORT";
@@ -314,6 +320,11 @@ bool hk_listbox::decode_action(QMimeSource* event)
         ftype=ot_query;
     }
     else
+    if (xtype=="VIEW")
+    {
+        ftype=ot_view;
+    }
+    else
     if (xtype=="FORM")
     {
         ftype=ot_form;
@@ -339,6 +350,8 @@ bool hk_listbox::execute_action(void)
     if (p_window->databasebutton->isChecked())actiontype=ot_database;
     else
     if (p_window->tablebutton->isChecked())actiontype=ot_table;
+        else
+        if (p_window->viewbutton->isChecked())actiontype=ot_view;
         else
         if (p_window->querybutton->isChecked())actiontype=ot_query;
             else
@@ -352,6 +365,11 @@ bool hk_listbox::execute_action(void)
             copy_table();
             break;
         }
+        case ot_view:
+        {
+            copy_view();
+            break;
+        }
         case ot_database:
         {
             copy_database();
@@ -432,6 +450,43 @@ bool hk_listbox::copy_table(void)
 }
 
 
+
+bool hk_listbox::copy_view(void)
+{
+    if (!p_window->datasource()) return false;
+    if (!p_window->datasource()->database()) return false;
+    hk_connection* con=find_connection();
+    hk_connection* newcon=p_window->datasource()->database()->connection()->drivermanager()->new_connection(xserver);
+    newcon->set_host(xhost);
+    newcon->set_tcp_port(localestring2uint(xtcp));
+    newcon->set_user(xuser);
+    if (con) newcon->set_password(con->password());
+    newcon->connect();
+    if (!newcon->is_connected())
+    {
+        delete newcon;
+        return false;
+    }
+    hk_database* db=newcon->new_database(xdbname);
+    hk_datasource* ds=NULL;
+    if (db)ds= db->new_view(xname);
+    p_cancelcopying=false;
+    p_progressdialog= new  QProgressDialog();
+    p_progressdialog->resize(300,p_progressdialog->height());
+    connect(p_progressdialog,SIGNAL(cancelled()),this,SLOT(copying_cancelled()));
+    p_progressdialog->show();
+    qApp->processEvents();
+    bool result= p_window->datasource()->database()->copy_view(ds,&hk_listbox::set_progress);
+    delete ds;
+    delete p_progressdialog;
+    p_progressdialog=NULL;
+    newcon->disconnect();
+    delete newcon;
+    return result;
+}
+
+
+
 void hk_listbox::copying_cancelled()
 {
     p_cancelcopying=true;
@@ -524,12 +579,14 @@ knodamainwindowbase::knodamainwindowbase( KMainWindow* parent, hk_kdeworkspace*
     knodamainwindowbaseLayout->setSpacing( 6 );
     knodamainwindowbaseLayout->setMargin( 11 );
     p_stablestring=i18n("&Tables");
+    p_sviewstring=i18n("&Views");
     p_squerystring=i18n("&Queries");
     p_sformstring=i18n("F&orms");
     p_sreportstring=i18n("&Reports");
 
     p_tablestring=i18n("Tables");
     p_querystring=i18n("Queries");
+    p_viewstring=i18n("Views");
     p_formstring=i18n("Forms");
     p_reportstring=i18n("Reports");
 
@@ -585,9 +642,9 @@ knodamainwindowbase::knodamainwindowbase( KMainWindow* parent, hk_kdeworkspace*
     querybutton->installEventFilter(this);
 
     reportbutton = new QRadioButton( ButtonGroup2, "reportbutton" );
-    reportbutton->setGeometry( QRect( 11, 121, 73, 19 ) );
+    reportbutton->setGeometry( QRect( 11, 147, 73, 19 ) );
     reportbutton->setText(  p_sreportstring );
-    ButtonGroup2->insert( reportbutton, 4 );
+    ButtonGroup2->insert( reportbutton, 5 );
     reportbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, reportbutton->sizePolicy().hasHeightForWidth() ) );
     reportbutton->setEnabled( FALSE );
     reportbutton->installEventFilter(this);
@@ -600,10 +657,18 @@ knodamainwindowbase::knodamainwindowbase( KMainWindow* parent, hk_kdeworkspace*
     tablebutton->setEnabled( FALSE );
     tablebutton->installEventFilter(this);
 
+    viewbutton = new QRadioButton( ButtonGroup2, "viewbutton" );
+    viewbutton->setGeometry( QRect( 11, 96, 73, 19 ) );
+    viewbutton->setText( p_sviewstring );
+    ButtonGroup2->insert( viewbutton, 3 );
+    viewbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, viewbutton->sizePolicy().hasHeightForWidth() ) );
+    viewbutton->setEnabled( FALSE );
+    viewbutton->installEventFilter(this);
+
     formbutton = new QRadioButton( ButtonGroup2, "formbutton" );
-    formbutton->setGeometry( QRect( 11, 96, 73, 19 ) );
+    formbutton->setGeometry( QRect( 11, 121, 73, 19 ) );
     formbutton->setText( p_sformstring );
-    ButtonGroup2->insert( formbutton, 3 );
+    ButtonGroup2->insert( formbutton, 4 );
     formbutton->setEnabled( FALSE );
     formbutton->installEventFilter(this);
 
@@ -617,6 +682,7 @@ knodamainwindowbase::knodamainwindowbase( KMainWindow* parent, hk_kdeworkspace*
     ButtonGroup2Layout->addWidget( databasebutton );
     ButtonGroup2Layout->addWidget( tablebutton );
     ButtonGroup2Layout->addWidget( querybutton );
+    ButtonGroup2Layout->addWidget( viewbutton );
     ButtonGroup2Layout->addWidget( formbutton );
     ButtonGroup2Layout->addWidget( reportbutton );
     Layout3->addWidget( ButtonGroup2 );
@@ -715,6 +781,11 @@ knodamainwindowbase::knodamainwindowbase( KMainWindow* parent, hk_kdeworkspace*
     p_passwordaction= new KAction(i18n("&Change password"),0,0,this,SLOT(newpassworddialog_selected()),p_parent->actionCollection(),"changepassword");
     p_copyaction= new KAction(i18n("&Copy"),"editcopy",CTRL+Key_C,this,SLOT(copy_clicked()),p_parent->actionCollection(),"copy");
     p_pasteaction= new KAction(i18n("&Paste"),"editpaste",CTRL+Key_V,this,SLOT(paste_clicked()),p_parent->actionCollection(),"paste");
+    
+    p_openlocaldbaction=  new KAction(i18n("Open &local database"),"fileopen",0,this,SLOT(slot_open_localdatabase()),p_parent->actionCollection(),"openlocaldb");   
+   
+   p_openlocaldbaction->setEnabled(false);
+
     QString rcfile=locate("data","knoda/knodaui.rc");
     if (rcfile.isNull())
     {
@@ -761,6 +832,7 @@ knodamainwindowbase::~knodamainwindowbase()
         p_connection=NULL;
         p_database=NULL;
     }
+    cerr <<"knodamainwindowbase::~knodamainwindowbase()"<<endl;
     delete p_newaction;
     delete p_alteraction;
     delete p_startaction;
@@ -773,6 +845,8 @@ knodamainwindowbase::~knodamainwindowbase()
     delete p_importaction;
     delete p_importcsvaction;
     delete p_passwordaction;
+    delete p_openlocaldbaction;
+
     delete Layout8;
     delete Layout3;
     delete knodamainwindowbaseLayout;
@@ -795,6 +869,8 @@ void knodamainwindowbase::set_connection(hk_connection*c )
     function_select(ot_database);
     databasebutton->setEnabled(true);
     p_passwordaction->setEnabled(p_connection->server_supports(hk_connection::SUPPORTS_CHANGE_PASSWORD));
+    p_openlocaldbaction->setEnabled(p_connection->server_supports(hk_connection::SUPPORTS_LOCAL_FILEFORMAT));  
+    
     if (!p_selecteddb.isEmpty())
     {
         p_database=p_connection->new_database();
@@ -809,6 +885,7 @@ void knodamainwindowbase::set_connection(hk_connection*c )
 	    p_database->set_name(p_selecteddb.local8Bit().data());
             set_datasource(p_database->new_resultquery());
             databasename->setText(p_selecteddb);
+	    open_autoform();
             if (!p_selectedform.isEmpty())
             {
                 hk_kdeform* w=new_form();
@@ -855,7 +932,13 @@ void knodamainwindowbase::exportcsv_clicked(void)
     if (p_database==NULL) return;
     hk_kdecsvexportdialog* e =new hk_kdecsvexportdialog(itemslist->currentText(),0,0,true);
     e->set_database(p_database);
-    e->set_usetablelist(tablebutton->isChecked());
+    if (querybutton->isChecked()) e->set_usequerylist();
+    else
+    if (viewbutton->isChecked()) e->set_useviewlist();
+    else
+     e->set_usetablelist();
+    
+    
     e->exec();
     delete e;
 }
@@ -869,7 +952,11 @@ void knodamainwindowbase::exportxml_clicked(void)
     if (p_database==NULL) return;
     hk_kdexmlexportdialog* e =new hk_kdexmlexportdialog(itemslist->currentText(),0,0,true);
     e->set_database(p_database);
-    e->set_usetablelist(tablebutton->isChecked());
+    if (querybutton->isChecked()) e->set_usequerylist();
+    else
+    if (viewbutton->isChecked()) e->set_useviewlist();
+    else
+     e->set_usetablelist();
     e->exec();
     delete e;
 }
@@ -898,6 +985,22 @@ void knodamainwindowbase::alter_clicked()
         return ;
     }
 
+    else
+    if (viewbutton->isChecked())
+    {
+//hk_kdetable ffnen
+
+        hk_kdequery* w=new_view();
+        hk_datasource* d=p_database->new_view();
+//   d->wanna_debug(true);
+        d->set_name(itemslist->text(itemslist->currentItem()).local8Bit().data());
+        w->set_datasource(d);
+        w->set_designmode();
+        if (open_maximized_windows())w->showMaximized();
+        else w->show();
+        return ;
+    }
+
     else
     if (querybutton->isChecked())
     {
@@ -1026,6 +1129,8 @@ void knodamainwindowbase::function_select(int f)
 #ifdef HK_DEBUG
     hkdebug("knodamainwindowbase::function_select");
 #endif
+    if (f==3) f=6;
+    else if (f>3) --f;
     if (p_connection==NULL) return;
     itemslist->clear();
     p_importaction->setEnabled(false);
@@ -1039,6 +1144,12 @@ void knodamainwindowbase::function_select(int f)
             vector<hk_string>* db=p_connection->dblist();
             vector<hk_string>::iterator it;
             itemslist->blockSignals(true);
+	    hk_url url=(hk_string)(databasename->text().local8Bit().data());
+	    if (!url.directory().empty())
+	    {
+	       itemslist->insertItem(databasename->text());
+	       itemslist->setCurrentItem(0);
+	    }   
             if (db!=NULL)
             {
                 int i=0;
@@ -1093,6 +1204,23 @@ void knodamainwindowbase::function_select(int f)
         }
         break;
 
+        case ot_view:
+        {
+            if (p_database==NULL) break;
+            TextLabel1->setText( p_viewstring );
+            p_exportaction->setEnabled(true);
+            vector<hk_string>* tb=p_database->viewlist();
+            vector<hk_string>::iterator it;
+            if (tb!=NULL)
+            {
+                for(it=tb->begin();it!=tb->end();it++)
+                {
+                    itemslist->insertItem(QString::fromLocal8Bit((*it).c_str()));
+                }
+            }
+        }
+        break;
+
         case ot_form:
         {
             if (p_database==NULL) break;
@@ -1147,6 +1275,8 @@ if (databasebutton->isChecked())actiontype=ot_database;
         else
         if (querybutton->isChecked())actiontype=ot_query;
             else
+        if (viewbutton->isChecked())actiontype=ot_view;
+            else
             if (formbutton->isChecked())actiontype=ot_form;
                 else actiontype=ot_report;
 function_select(actiontype);
@@ -1208,6 +1338,19 @@ void knodamainwindowbase::new_clicked()
         return ;
     }
 
+    else
+    if (viewbutton->isChecked())
+    {
+        if (p_database==NULL) return;
+        hk_kdequery* w=new_query();
+        hk_datasource* d=p_database->new_view();
+        w->set_datasource(d);
+        w->set_designmode();
+        if (open_maximized_windows())w->showMaximized();
+        else w->show();
+        return ;
+    }
+
     else
     if (formbutton->isChecked())
     {
@@ -1267,6 +1410,20 @@ void knodamainwindowbase::start_clicked()
         return ;
     }
     else
+    if (viewbutton->isChecked())
+    {
+        hk_kdequery* w=new_query();
+        hk_datasource* d=p_database->new_view();
+        d->set_name(itemslist->text(itemslist->currentItem()).local8Bit().data());
+        w->set_datasource(d);
+        w->set_use_qbe(false,false);
+
+        if (open_maximized_windows())w->showMaximized();
+        else w->show();
+        w->set_viewmode();
+        return ;
+    }
+    else
     if (formbutton->isChecked())
     {
         hk_kdeform* w=new_form();
@@ -1323,6 +1480,8 @@ void knodamainwindowbase::itemslist_clicked(QListBoxItem* item)
                     ;
                 }
                 else databasename->setText("");
+		open_autoform();
+
                 set_datasource(p_database->new_resultquery());
             }
         }
@@ -1332,6 +1491,21 @@ void knodamainwindowbase::itemslist_clicked(QListBoxItem* item)
 }
 
 
+void knodamainwindowbase::open_autoform()
+{
+  if (p_database && p_database->form_exists(autoform.local8Bit().data()))
+{
+   hk_kdeform* w=new_form();
+   w->set_database(p_database);
+   w->load_form(autoform.local8Bit().data());
+    show();
+   if (open_maximized_windows())w->showMaximized();
+    else w->show();
+   w->set_viewmode();
+}
+
+}
+
 void    knodamainwindowbase::list_changes(listtype type)
 {
 #ifdef HK_DEBUG
@@ -1344,6 +1518,8 @@ void    knodamainwindowbase::list_changes(listtype type)
         break;
         case lt_form : f=ot_form;
         break;
+        case lt_view : f=ot_view;
+        break;
         case lt_report : f=ot_report;
         break;
         case lt_table : f=ot_table;
@@ -1507,12 +1683,14 @@ void knodamainwindowbase::set_minimalbuttons()
     {
         tablebutton->setEnabled(false);
         querybutton->setEnabled(false);
+	viewbutton->setEnabled(false);
         formbutton->setEnabled(false);
         reportbutton->setEnabled(false);
     }
     else
     {
         tablebutton->setEnabled(true);
+        viewbutton->setEnabled(p_connection->server_supports(hk_connection::SUPPORTS_VIEWS));
         querybutton->setEnabled(true);
         formbutton->setEnabled(true);
         reportbutton->setEnabled(true);
@@ -1619,6 +1797,12 @@ return new hk_kdequery(p_workspace?p_workspace->p_workspace:NULL,0,WDestructiveC
 }
 
 
+hk_kdequery* knodamainwindowbase::new_view(void)
+{
+return new hk_kdequery(p_workspace?p_workspace->p_workspace:NULL,0,WDestructiveClose);
+}
+
+
 
 bool knodamainwindowbase::eventFilter(QObject* object, QEvent* event)
 {
@@ -1651,3 +1835,38 @@ void knodamainwindowbase::objecthandler_clicked(void)
   delete h;
 }
 
+
+void knodamainwindowbase::slot_open_localdatabase(void)
+{
+  if (!p_connection) return;
+  QStringList l;
+  QString sqlite="application/x-hk_classes-"; sqlite+=QString::fromLocal8Bit(p_connection->drivername().c_str()); 
+  l.append(sqlite);
+  l.append("all/allfiles");
+KFileDialog* d=new KFileDialog(QString::null,QString::null,this,0,true);
+d->setMimeFilter(l,sqlite);
+d->exec();
+QString filename=d->selectedFile();
+  if (!filename.isNull())
+  {
+     itemslist->blockSignals(true);
+     itemslist->insertItem(filename);
+     itemslist->blockSignals(false);
+  }   
+     databasename->setText(filename);
+
+     
+  if (!p_database)p_database=p_connection->new_database();
+  if (p_database!=NULL)
+    {
+     p_database->set_usewidgetparent(p_workspace);
+     p_database->set_name(filename.local8Bit().data());
+    } 
+     
+     
+     
+     function_select(ot_database);
+delete d;
+}
+
+
